// Conversion.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef CONVERSION_H
#define CONVERSION_H
///////////////////////////////////////////////////////////
const Uint16 WILDCARD_QUES = L'?';
const Uint16 WILDCARD_STAR = L'_';
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
const Uint16 UNICODE_ENGLISH_START =  0x0021; // 012... (starts w/ '!')
//const Uint16 UNICODE_ENGLISH_START =  0x0030; // 012...
const Uint16 UNICODE_ENGLISH_END =    0x007A; // z
const Uint16 UNICODE_HIRAGANA_START = 0x3040; // actually 0x3040 is the reserved start of it.
const Uint16 UNICODE_HIRAGANA_END =	  0x309F; // (0x3097-0x3098 not used)
const Uint16 UNICODE_KATAKANA_START = 0x30A0;
const Uint16 UNICODE_KATAKANA_END =   0x30FF;
///////////////////////////////////////////////////////////
Uint16 KANA_OFFSET = 0x0060;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsKatakana( Uint16 c ) {
	if( c >= UNICODE_KATAKANA_START && c <= UNICODE_KATAKANA_END ) {
		return( true );
	}
	return( false );
}
bool IsHiragana( Uint16 c ) {
	if( c >= UNICODE_HIRAGANA_START && c <= UNICODE_HIRAGANA_END ) {
		if( c >= 0x3041 && c <= 0x3096 ) {  // no extra characters.
			return( true );
		}
	}
	return( false );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsKatakana( wstring &str ) {
	int length = str.size();
	for( int i = 0; i < length; i++ ) {
		if( !IsKatakana( str[i] ) ) {
			return( false );
		}
	}
	return( true );
}
bool IsHiragana( wstring &str ) {
	int length = str.size();
	for( int i = 0; i < length; i++ ) {
		if( !IsHiragana( str[i] ) ) {
			return( false );
		}
	}
	return( true );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsKana( wstring &str ) {
	for( int i = 0; i < str.size(); i++ ) {
		if( !(IsHiragana(str[i])||IsKatakana(str[i])) ) {
			return(false);
		}
	}
	return(true);
}
///////////////////////////////////////////////////////////
bool IsKana_WildcardsOkay( wstring &str ) {
	for( int i = 0; i < str.size(); i++ ) {
		if( !(IsHiragana(str[i])||IsKatakana(str[i])) ) {
			if( str[i] == WILDCARD_QUES || str[i] == WILDCARD_STAR ) {
			} else {
				return(false);
			}
		}
	}
	return(true);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
Uint16 ToKatakana( Uint16 c ) {
	if( IsHiragana(c) ) {
		int offset = c - UNICODE_HIRAGANA_START;
		c = UNICODE_KATAKANA_START+offset;
	}
	return( c );
}
Uint16 ToHiragana( Uint16 c ) {
	if( IsKatakana(c) ) {
		int offset = c - UNICODE_KATAKANA_START;
		c = UNICODE_HIRAGANA_START+offset;
	}
	return( c );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring ToKatakana( wstring &str ) {
	wstring katakana = str;
	for( int i = 0; i < str.length(); i++ ) {
		katakana[i] = ToKatakana( str[i] );
	}
	return(katakana);
}
wstring ToHiragana( wstring &str ) {
	wstring hiragana = str;
	for( int i = 0; i < str.length(); i++ ) {
		hiragana[i] = ToHiragana( str[i] );
	}
	return(hiragana);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool StringsAreEqual_IgnoreKana( wstring &a, wstring &b ) {
	if( a.length() == b.length() ) {
		int len = a.length();
		for( int i = 0; i < len; i++ ) {
			if( ToHiragana(a[i]) == ToHiragana(b[i]) && ToKatakana(a[i]) == ToKatakana(b[i]) ) {
			} else {
				return( false );
			}
		}
	} else {
		return( false );
	}
	return( true );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsVowel( Uint16 c ) {
	if( c==L'a' || c==L'i' || c==L'u' || c==L'e' || c==L'o' ) {
		return(true);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsRomaji( Uint16 c ) {
	if( c >= UNICODE_ENGLISH_START && c <= UNICODE_ENGLISH_END ) {
		return(true);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsUpper( Uint16 u ) {
	if( u >= 0x0041 && u <= 0x005A ) {
		return(true);
	}
	return(false);
}
///////////////////////////////////////////////////////////
bool IsLower( Uint16 u ) {
	if( u >= 0x0061 && u <= 0x007A ) {
		return(true);
	}
	return(false);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool IsLower( wstring &text ) {
	for( int i = 0; i < text.size(); i++ ) {
		if( IsUpper(text[i]) ) {
			return(false);
		}
	}
	return(true);
}
///////////////////////////////////////////////////////////
bool IsUpper( wstring &text ) {
	for( int i = 0; i < text.size(); i++ ) {
		if( IsLower(text[i]) ) {
			return(false);
		}
	}
	return(true);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
Uint16 ToLower( Uint16 u ) {
	if( IsUpper(u) ) {
		u = u + (L'a'-L'A');
	}
	return(u);
}
///////////////////////////////////////////////////////////
Uint16 ToUpper( Uint16 u ) {
	if( IsLower(u) ) {
		u = u - (L'a'-L'A');
	}
	return(u);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring ToLower( wstring text ) {
	for( int i = 0; i < text.size(); i++ ) {
		if( IsUpper(text[i]) ) {
			text[i] = text[i] + (L'a'-L'A');
		}
	}
	return(text);
}
///////////////////////////////////////////////////////////
wstring ToUpper( wstring text ) {
	for( int i = 0; i < text.size(); i++ ) {
		if( IsLower(text[i]) ) {
			text[i] = text[i] - (L'a'-L'A');
		}
	}
	return(text);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// Capital letters denote smaller kana:
Uint16 RomajiToKana_Single( wstring c ) {
	int offset = !IsLower(c)?KANA_OFFSET:0x0000;
	c = ToLower(c);
	Uint16 u = 0;
		  if( c==L"la" ) {		u = 0x3041 + offset;	}
	else if( c==L"a" )	{		u = 0x3042 + offset;	}
	else if( c==L"li" ) {		u = 0x3043 + offset;	}
	else if( c==L"i" ) {			u = 0x3044 + offset;	}
	else if( c==L"lu" ) {		u = 0x3045 + offset;	}
	else if( c==L"u" ) {			u = 0x3046 + offset;	} 
	else if( c==L"le" ) {		u = 0x3047 + offset;	}
	else if( c==L"e" ) {			u = 0x3048 + offset;	}
	else if( c==L"lo" ) {		u = 0x3049 + offset;	}
	else if( c==L"o" ) {			u = 0x304A + offset;	}
	else if( c==L"ka" ) {		u = 0x304B + offset;	}
	else if( c==L"ga" ) {		u = 0x304C + offset;	}
	else if( c==L"ki" ) {		u = 0x304D + offset;	}
	else if( c==L"gi" ) {		u = 0x304E + offset;	}
	else if( c==L"ku" ) {		u = 0x304F + offset;	}
	else if( c==L"gu" ) {		u = 0x3050 + offset;	}
	else if( c==L"ke" ) {		u = 0x3051 + offset;	}
	else if( c==L"ge" ) {		u = 0x3052 + offset;	}
	else if( c==L"ko" ) {		u = 0x3053 + offset;	}
	else if( c==L"go" ) {		u = 0x3054 + offset;	}
	else if( c==L"sa" ) {		u = 0x3055 + offset;	}
	else if( c==L"za" ) {		u = 0x3056 + offset;	}
	else if( c==L"si" ) {		u = 0x3057 + offset;	}
	else if( c==L"zi" ) {		u = 0x3058 + offset;	}
	else if( c==L"su" ) {		u = 0x3059 + offset;	}
	else if( c==L"zu" ) {		u = 0x305A + offset;	}
	else if( c==L"se" ) {		u = 0x305B + offset;	}
	else if( c==L"ze" ) {		u = 0x305C + offset;	}
	else if( c==L"so" ) {		u = 0x305D + offset;	}
	else if( c==L"zo" ) {		u = 0x305E + offset;	}
	else if( c==L"ta" ) {		u = 0x305F + offset;	}
	else if( c==L"da" ) {		u = 0x3060 + offset;	}
	else if( c==L"ti" ) {		u = 0x3061 + offset;	}
	else if( c==L"di" ) {		u = 0x3062 + offset;	}
	else if( c==L"ltu" ) {		u = 0x3063 + offset;	}
	else if( c==L"tu" ) {		u = 0x3064 + offset;	}
	else if( c==L"du" ) {		u = 0x3065 + offset;	}
	else if( c==L"te" ) {		u = 0x3066 + offset;	}
	else if( c==L"de" ) {		u = 0x3067 + offset;	}
	else if( c==L"to" ) {		u = 0x3068 + offset;	}
	else if( c==L"do" ) {		u = 0x3069 + offset;	}
	else if( c==L"na" ) {		u = 0x306A + offset;	}
	else if( c==L"ni" ) {		u = 0x306B + offset;	}
	else if( c==L"nu" ) {		u = 0x306C + offset;	}
	else if( c==L"ne" ) {		u = 0x306D + offset;	}
	else if( c==L"no" ) {		u = 0x306E + offset;	}
	else if( c==L"ha" ) {		u = 0x306F + offset;	}
	else if( c==L"ba" ) {		u = 0x3070 + offset;	}
	else if( c==L"pa" ) {		u = 0x3071 + offset;	}
	else if( c==L"hi" ) {		u = 0x3072 + offset;	}
	else if( c==L"bi" ) {		u = 0x3073 + offset;	}
	else if( c==L"pi" ) {		u = 0x3074 + offset;	}
	else if( c==L"hu" ) {		u = 0x3075 + offset;	}
	else if( c==L"bu" ) {		u = 0x3076 + offset;	}
	else if( c==L"pu" ) {		u = 0x3077 + offset;	}
	else if( c==L"he" ) {		u = 0x3078 + offset;	}
	else if( c==L"be" ) {		u = 0x3079 + offset;	}
	else if( c==L"pe" ) {		u = 0x307A + offset;	}
	else if( c==L"ho" ) {		u = 0x307B + offset;	}
	else if( c==L"bo" ) {		u = 0x307C + offset;	}
	else if( c==L"po" ) {		u = 0x307D + offset;	}
	else if( c==L"ma" ) {		u = 0x307E + offset;	}
	else if( c==L"mi" ) {		u = 0x307F + offset;	}
	else if( c==L"mu" ) {		u = 0x3080 + offset;	}
	else if( c==L"me" ) {		u = 0x3081 + offset;	}
	else if( c==L"mo" ) {		u = 0x3082 + offset;	}
	else if( c==L"lya" ) {		u = 0x3083 + offset;	}
	else if( c==L"ya" ) {		u = 0x3084 + offset;	}
	else if( c==L"lyu" ) {		u = 0x3085 + offset;	}
	else if( c==L"yu" ) {		u = 0x3086 + offset;	}
	else if( c==L"lyo" ) {		u = 0x3087 + offset;	}
	else if( c==L"yo" ) {		u = 0x3088 + offset;	}
	else if( c==L"ra" ) {		u = 0x3089 + offset;	}
	else if( c==L"ri" ) {		u = 0x308A + offset;	}
	else if( c==L"ru" ) {		u = 0x308B + offset;	}
	else if( c==L"re" ) {		u = 0x308C + offset;	}
	else if( c==L"ro" ) {		u = 0x308D + offset;	}
	else if( c==L"lwa" ) {		u = 0x308E + offset;	}
	else if( c==L"wa" ) {		u = 0x308F + offset;	}
	else if( c==L"wi" ) {		u = 0x3090 + offset;	}
	else if( c==L"we" ) {		u = 0x3091 + offset;	}
	else if( c==L"wo" ) {		u = 0x3092 + offset;	}
	else if( c==L"n" ) {			u = 0x3093 + offset;	} else if( c==L"n'" ) {		u = 0x3093 + offset;	}
	else if( c==L"vu" ) {		u = 0x3094 + offset;	}
	else if( c==L"lka" ) {		u = 0x3095 + offset;	}
	else if( c==L"lke" ) {		u = 0x3096 + offset;	}
	// Katakana only:
	else if( c==L"va" ) {		u = 0x30F7;  }
	else if( c==L"vi" ) {		u = 0x30F8;  }
	else if( c==L"ve" ) {		u = 0x30F9;  }
	else if( c==L"vo" ) {		u = 0x30FA;  }
	else if( c==L"*" ) {			u = 0x30FB;  }
	else if( c==L"-" ) {			u = 0x30FC;  }

	return( u );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring RomajiToKana( wstring text ) {
	wstring kana;

	Uint16 offset = !IsLower(text)?KANA_OFFSET:0x0000;

	text = ToLower(text);

	// Go over the kya patterns:
	if( text.size() == 3 && text[1] == L'y' && text[0] != L'l' ) {
		wstring ipart;
			ipart = text[0];
			ipart += L'i';
			//MessageBox(NULL,ipart.c_str(),text.c_str(),MB_OK );
		wstring ypart;
			ypart = L'l';
			ypart += text[1];
			ypart += text[2];
		kana = RomajiToKana_Single( ipart ) + offset;
		kana += RomajiToKana_Single( ypart ) + offset;
	} else {
		Uint16 c = RomajiToKana_Single( text ) + offset;
		kana = c;
	}

	return(kana);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
wstring RomajiStringToKana( wstring text ) {
#ifdef DEBUG_CONVERSION
	static bool first = true;
	FILE *fcon = NULL;
	if( first ) {
		fcon = fopen("out.conversion.txt","wb");
		fwprintf(fcon,L"%c",0xFEFF );
		first = false;
	} else {
		fcon = fopen("out.conversion.txt","ab");
	}

	fwprintf(fcon,L"Conversion: %s\n",text.c_str() );
#endif


	wstring kana = L"";

	wstring temp;
	bool katakana = false;
	for( int i = 0; i < text.size(); i++ ) {
		Uint16 c = text[i];

		if( IsUpper(c) ) {
			katakana = true;
			c = ToLower(c);
		}

		//fwprintf(fcon,L"romaji: <<%c>> (0x%04X) %skatakana\n",c,c,katakana?L"":L"NOT ");

		// If it's ROMAji, then we're gonna convert it to kana:
		// _ is a special case.
		if( IsRomaji(c) && c != L'_' && c != L'?' ) {
			temp += c;

			// If it's NOT a vowel, then we're going to make sure it's okay.
			if( !IsVowel(c) ) {
				if( temp.size() == 2 ) {
					// Check for 'double consonants':
					if( temp[0] == temp[1] ) {
						// Check for n's:
						// For example, break the 'nn' in konniti to 'n' + 'n'...
						if( temp[0] == L'n' ) {
							if( katakana ) {  kana += RomajiToKana(L"N");	}
							else {	kana += RomajiToKana(L"n");	}
							temp = L'n';
						}
						// Check for tsu's:
						// In other words, every double consonant that isn't an 'n' is an 'ltu'...
						else {
							if( katakana ) {	kana += RomajiToKana(L"LTU");  }
							else {	kana += RomajiToKana(L"ltu");  }
							temp = temp[1];
						}
					} else {
					// If the consonants are NOT the same:
						// Check for n's:
						// For example, break the 'np' in konpyuta- to 'n' + 'p'...
						if( temp[0] == L'n' && temp[1] != L'\'' ) {
							if( katakana ) {  kana += RomajiToKana(L"N");	}
							else {	kana += RomajiToKana(L"n");	}
							temp = temp[1];
						}
					}
				}// (size==2)
				// Do (if size==1) AFTER ==2 because it may get SMALLER after that one:
				if( temp.size() == 1 ) {
						  /*if( temp == L"j" ) {  temp = L"zy";  }
					else*/ if( temp == L"f" ) {	temp = L'h';  }
				} // (size==1)
			}// (!isvowel)

			// If it's a vowel or a special character, then it's time to get the kana for it:
			if( IsVowel(c) || c == L'\'' || c == L'-' || c == L'*' || i == text.size()-1 ) {
#ifdef DEBUG_CONVERSION
				fwprintf(fcon,L"   temp: <<%s>> ... ",temp.c_str() );
#endif



				// Even if it's a vowel, fix up the shoddy ROMAji:
				if( temp.size() == 2 ) {
						  if( temp==L"ja" ) {	temp=L"zya";	}
					else if( temp==L"ji" ) {	temp=L"zi";	}
					else if( temp==L"ju" ) {	temp=L"zyu";	}
					else if( temp==L"jo" ) {	temp=L"zyo";	}
				}
				else if( temp.size() == 3) {
						  if( temp==L"shi" ) {	temp = L"si";  }
					else if( temp==L"sha" ) {	temp = L"sya";	}
					else if( temp==L"shu" ) {	temp = L"syu";	}
					else if( temp==L"sho" ) {	temp = L"syo";	}
					else if( temp==L"chi" ) {	temp = L"ti";	}
					else if( temp==L"cha" ) {	temp = L"tya";	}
					else if( temp==L"chu" ) {	temp = L"tyu";	}
					else if( temp==L"cho" ) {	temp = L"tyo";	}
				}// (size==3)
				else if( temp.size() == 4 ) {
					if( temp==L"ltsu" ) {	temp = L"ltu";	}
				}




				if( katakana ) {	temp = ToUpper(temp);	}
#ifdef DEBUG_CONVERSION
				fwprintf(fcon,L" <<%s>> ... ",temp.c_str() );
#endif
				temp = RomajiToKana( temp );
				if( temp[0] != 0 ) {
#ifdef DEBUG_CONVERSION
					fwprintf(fcon,L"<<%s>>\n",temp.c_str() );
#endif
					kana += temp;
				} else {
#ifdef DEBUG_CONVERSION
					fwprintf(fcon,L"(0)\n");
#endif
				}

				temp = L"";
				katakana = false;
			}
		}// (isromaji)
		else {
			if( IsHiragana(c)||IsKatakana(c) ) {
				kana += c;
			} else {
			// It's probably a kanji character; add it, too:
				kana += c;
			}
		}
	}// for

#ifdef DEBUG_CONVERSION
	fwprintf(fcon,L"Converted to: <<%s>> (%i)\n\n",kana.c_str(), kana.size() );
	fclose(fcon);
#endif

	return(kana);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////