// CopyPaste.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef COPYPASTE_H
#define COPYPASTE_H
///////////////////////////////////////////////////////////
// For the combobox and sturctures:
#include "Search_Rebar.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*		This function copies the specified text 'searchtext'
	to the clipboard based on the currently selected encoding
	as specified in the encoding combobox.

	* Returns:
	0		Success.
	-1		Failure.
*/
int CopyTextUsingComboboxEncoding( wstring &searchtext ) {
	/////////////////////////////////////
	///   STEP 1: Get the encoding.   ///
	/////////////////////////////////////
	// Get the encoding in the combobox (index):
	int encoding = SendMessage( hwnd_searchrebar_combobox_encode,CB_GETCURSEL, 0,0 );
	int index = -1;
	for( int i = 0; i < encoding_array_size; i++ ) {
		if( encoding_array[i].combobox_index == encoding ) {	index = i;	break;	}
	}

	/////////////////////////////////////////////
	///   STEP 2: Actually convert & paste.   ///
	/////////////////////////////////////////////
	// As long as the index is valid, convert it:
	if( index >= 0 ) {
		
		/* Note: Index 1 is 'Normal Unicode' (no processing).
			If this is selected, then we don't do *any* conversions, and all
			we do is paste in the Windows Unicode format CF_UNICODETEXT. */
		if( index == 1 ) {
			// Open the clipboard:
			OpenClipboard( NULL );

			// Allocate a global memory object for the text. 
			HGLOBAL hglbCopy = GlobalAlloc(GMEM_MOVEABLE, (searchtext.size()+1)*sizeof(Uint16) );
			if(hglbCopy != NULL) {
				// Clear what was there:
				EmptyClipboard();
				// Lock the handle and copy the text to the buffer. 
				Uint16 *lptstrCopy = (Uint16*)GlobalLock( hglbCopy ); 
				memcpy( lptstrCopy, searchtext.c_str(), searchtext.size()*sizeof(Uint16) );
					lptstrCopy[searchtext.size()] = L'\0';    // null character
				GlobalUnlock(hglbCopy); 

				// Place the handle on the clipboard. 
				SetClipboardData( CF_UNICODETEXT, hglbCopy );
			}
			CloseClipboard();
		} else {
		/*    In this case, we need to convert the text to the format as specified
			in the combobox.  So, first we do that, and then we copy it. */
			// Open the clipboard:
			OpenClipboard( NULL );

			// Convert the text, first:
			string strenc;
				ConvertEncoding( (char*)searchtext.c_str(), searchtext.size()*sizeof(Uint16), "UTF-16LE", encoding_array[i].iconv_string, strenc );

			// Allocate a global memory object for the text.
			HGLOBAL hglbCopy = GlobalAlloc( GMEM_MOVEABLE, strenc.size()+1 );
			if(hglbCopy != NULL) {
				// Clear what was there:
				EmptyClipboard();
				// Lock the handle and copy the text to the buffer. 
				char *lptstrCopy = (char*)GlobalLock( hglbCopy );
				memcpy( lptstrCopy, (char*)strenc.c_str(), strenc.size() );
					lptstrCopy[strenc.size()] = '\0';    // null character
				GlobalUnlock( hglbCopy ); 

				// Place the handle on the clipboard. 
				SetClipboardData( CF_TEXT, hglbCopy );
			}
			CloseClipboard();
		}
	} else {
		fprintf(g_flog,"CopyTextUsingComboboxEncoding(): ERROR: index out of bounds (index: %i).\n",index );
		return(-1);
	}
	return(0);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
