// Dialog_Options.h | by Doug Manley (2004.07.04)
///////////////////////////////////////////////////////////
#ifndef DIALOG_OPTIONS_H
#define DIALOG_OPTIONS_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
enum {
	OPTIONS_JLOOKUP,
	OPTIONS_MISC,
	OPTIONS_DICTIONARIES,
	OPTIONS_SEARCHWINDOW,
	OPTIONS_KANJIWINDOW,
	OPTIONS_RESULTWINDOW,
	OPTIONS_MAX
};

HWND hwnd_options_right[OPTIONS_MAX];
int options_right_current;

int options_right_x, options_right_y;
///////////////////////////////////////////////////////////
#include "Dialog_Options_JLookUp.h"
#include "Dialog_Options_Misc.h"
#include "Dialog_Options_Dictionaries.h"
#include "Dialog_Options_SearchWindow.h"
#include "Dialog_Options_ResultWindow.h"
#include "Dialog_Options_KanjiWindow.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
BOOL CALLBACK DlgProc_Options( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {
	switch( message ) {
		case WM_INITDIALOG: {
		/* wParam
				Handle to the control to receive the default keyboard focus. The system assigns the default keyboard focus only if the dialog box procedure returns TRUE. 
			lParam
				Specifies additional initialization data. This data is passed to the system as the lParam parameter in a call to the CreateDialogIndirectParam, CreateDialogParam, DialogBoxIndirectParam, or DialogBoxParam function used to create the dialog box. For property sheets, this parameter is a pointer to the PROPSHEETPAGE structure used to create the page. This parameter is zero if any other dialog box creation function is used. 
			Return: TRUE if we want to set keyboard focus to wParam, FALSE otherwise.
		*/

			// Figure out where the right-hand-side window will go:
			RECT rcWin;
				GetWindowRect( hWnd, &rcWin );
			RECT rc;
				GetWindowRect( GetDlgItem(hWnd,IDC_OPTIONTREE), &rc );
			options_right_x = rc.right-rcWin.left;
			options_right_y = 0;//rc.top-rcWin.top; (for some reason, this just won't work)


			// Create the child dialog boxes.
			// These are the ones that go on the right:
			options_right_current = OPTIONS_JLOOKUP;
			hwnd_options_right[OPTIONS_JLOOKUP] = CreateDialog_Options_JLookUp( hWnd );
			hwnd_options_right[OPTIONS_MISC] = CreateDialog_Options_Misc( hWnd );
			hwnd_options_right[OPTIONS_DICTIONARIES] = CreateDialog_Options_Dictionaries( hWnd );
			hwnd_options_right[OPTIONS_SEARCHWINDOW] = CreateDialog_Options_SearchWindow( hWnd );
			hwnd_options_right[OPTIONS_KANJIWINDOW] = CreateDialog_Options_KanjiWindow( hWnd );
			hwnd_options_right[OPTIONS_RESULTWINDOW] = CreateDialog_Options_ResultWindow( hWnd );

			for( int i = 0; i < OPTIONS_MAX; i++ ) {
				ShowWindow( hwnd_options_right[i], SW_HIDE );
				SetWindowPos( hwnd_options_right[i], 0, options_right_x,options_right_y, 0,0, SWP_NOZORDER|SWP_NOSIZE );
			}



			HTREEITEM ti_parent, ti_child;
			{
				TVINSERTSTRUCT tvinsert;
					tvinsert.hParent = NULL; // top most level Item
					tvinsert.hInsertAfter = TVI_ROOT; // root level item attribute.
					// TVITEM:
					tvinsert.item.mask = TVIF_TEXT|TVIF_PARAM;
					tvinsert.item.pszText =L"JLookUp Options";
					tvinsert.item.lParam = OPTIONS_JLOOKUP;
				ti_parent = (HTREEITEM)SendDlgItemMessage( hWnd, IDC_OPTIONTREE, TVM_INSERTITEM, 0, (LPARAM)&tvinsert );
			}


			{
				TVINSERTSTRUCT tvinsert;
					tvinsert.hParent = ti_parent; // top most level Item
					tvinsert.hInsertAfter = TVI_ROOT; // root level item attribute.
					// TVITEM:
					tvinsert.item.mask = TVIF_TEXT|TVIF_PARAM;
					tvinsert.item.pszText =L"Misc. Options";
					tvinsert.item.lParam = OPTIONS_MISC;
				ti_child = (HTREEITEM)SendDlgItemMessage( hWnd, IDC_OPTIONTREE, TVM_INSERTITEM, 0, (LPARAM)&tvinsert );
			}


			{
				TVINSERTSTRUCT tvinsert;
					tvinsert.hParent = ti_parent; // top most level Item
					tvinsert.hInsertAfter = TVI_ROOT; // root level item attribute.
					// TVITEM:
					tvinsert.item.mask = TVIF_TEXT|TVIF_PARAM;
					tvinsert.item.pszText =L"Dictionaries";
					tvinsert.item.lParam = OPTIONS_DICTIONARIES;
				ti_child = (HTREEITEM)SendDlgItemMessage( hWnd, IDC_OPTIONTREE, TVM_INSERTITEM, 0, (LPARAM)&tvinsert );
			}
			
			{
				TVINSERTSTRUCT tvinsert;
					tvinsert.hParent = ti_parent; // top most level Item
					tvinsert.hInsertAfter = TVI_ROOT; // root level item attribute.
					// TVITEM:
					tvinsert.item.mask = TVIF_TEXT|TVIF_PARAM;
					tvinsert.item.pszText =L"Search Window";
					tvinsert.item.lParam = OPTIONS_SEARCHWINDOW;
				ti_child = (HTREEITEM)SendDlgItemMessage( hWnd, IDC_OPTIONTREE, TVM_INSERTITEM, 0, (LPARAM)&tvinsert );
			}

			{
				TVINSERTSTRUCT tvinsert;
					tvinsert.hParent = ti_parent; // top most level Item
					tvinsert.hInsertAfter = TVI_ROOT; // root level item attribute.
					// TVITEM:
					tvinsert.item.mask = TVIF_TEXT|TVIF_PARAM;
					tvinsert.item.pszText =L"Kanji Window";
					tvinsert.item.lParam = OPTIONS_KANJIWINDOW;
				ti_child = (HTREEITEM)SendDlgItemMessage( hWnd, IDC_OPTIONTREE, TVM_INSERTITEM, 0, (LPARAM)&tvinsert );
			}

			{
				TVINSERTSTRUCT tvinsert;
					tvinsert.hParent = ti_parent; // top most level Item
					tvinsert.hInsertAfter = TVI_ROOT; // root level item attribute.
					// TVITEM:
					tvinsert.item.mask = TVIF_TEXT|TVIF_PARAM;
					tvinsert.item.pszText =L"Result Window";
					tvinsert.item.lParam = OPTIONS_RESULTWINDOW;
				ti_child = (HTREEITEM)SendDlgItemMessage( hWnd, IDC_OPTIONTREE, TVM_INSERTITEM, 0, (LPARAM)&tvinsert );
			}


			// Expand the tree from the top:
			SendDlgItemMessage( hWnd, IDC_OPTIONTREE, TVM_EXPAND, TVE_EXPAND, (LPARAM)ti_parent );

			// Select the top one:
			SendDlgItemMessage( hWnd, IDC_OPTIONTREE, TVM_SELECTITEM, TVGN_CARET, (LPARAM)ti_parent );
			
			SetFocus( GetDlgItem(hWnd,IDC_OPTIONTREE) );

			return(TRUE);
		}
		case WM_COMMAND: {
			switch( LOWORD(wParam) ) {
				case IDOK: {
					EndDialog( hWnd, IDOK );
					break;
				}
				case IDCANCEL: {
					EndDialog( hWnd, IDCANCEL );
					break;
            }
			}
			return(TRUE);
		}
		case WM_NOTIFY: {
			NMHDR *hdr = (NMHDR*)lParam;
			if( hdr->idFrom == IDC_OPTIONTREE ) {
      		NMTREEVIEW *nmtv = (NMTREEVIEW*)lParam;
				switch( nmtv->hdr.code ) {
#ifdef VERSION_NT
					case TVN_SELCHANGED: {
#else
					case TVN_SELCHANGEDW:
					case TVN_SELCHANGEDA: {
#endif
						// Hide the window that's up now:
						ShowWindow( hwnd_options_right[options_right_current], SW_HIDE );

						// A new window is up:
						options_right_current = nmtv->itemNew.lParam;
						ShowWindow( hwnd_options_right[options_right_current], SW_SHOW );
						break;
					}
            }
			}
         return(TRUE);
      }//*/
		case WM_DESTROY: {
			for( int i = 0; i < OPTIONS_MAX; i++ ) {
				DestroyWindow( hwnd_options_right[i] );
				hwnd_options_right[i] = NULL;
			}
			return(TRUE);
		}
   }
	// Returning FALSE causes it to do the default processing for the message.
   return(FALSE); 
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void CreateDialog_Options( HWND hwndParent ) {
	int result = DialogBox(
		g_hinst,
		MAKEINTRESOURCE( IDD_OPTIONS ), 
		hwndParent,
		(DLGPROC)DlgProc_Options
	);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
