// Dialog_Options_KanjiWindow.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef DIALOG_OPTIONS_KANJIWINDOW_H
#define DIALOG_OPTIONS_KANJIWINDOW_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
Color options_color_kanji_bg( color_kanji_bg1 );
HBRUSH hbrush_kanji = CreateSolidBrush( options_color_kanji_bg.ToCOLORREF() );
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
BOOL CALLBACK DlgProc_Options_KanjiWindow( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {
	switch( message ) {
		case WM_INITDIALOG: {
		/* wParam
				Handle to the control to receive the default keyboard focus. The system assigns the default keyboard focus only if the dialog box procedure returns TRUE. 
			lParam
				Specifies additional initialization data. This data is passed to the system as the lParam parameter in a call to the CreateDialogIndirectParam, CreateDialogParam, DialogBoxIndirectParam, or DialogBoxParam function used to create the dialog box. For property sheets, this parameter is a pointer to the PROPSHEETPAGE structure used to create the page. This parameter is zero if any other dialog box creation function is used. 
			Return: TRUE if we want to set keyboard focus to wParam, FALSE otherwise.
		*/

			// Set the text of the font boxes:
			Uint16 str[1024];
				swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Kanji\\Fonts\\Kanji\\Font"), registry.GetINT(".\\Kanji\\Fonts\\Kanji\\Size") );
			SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_KANJI, WM_SETTEXT, 0, (LPARAM)str );
				swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Kanji\\Fonts\\On\\Font"), registry.GetINT(".\\Kanji\\Fonts\\On\\Size") );
			SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_ON, WM_SETTEXT, 0, (LPARAM)str );
			SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_KUN, WM_SETTEXT, 0, (LPARAM)str );
				swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Kanji\\Fonts\\Meaning\\Font"), registry.GetINT(".\\Kanji\\Fonts\\Meaning\\Size") );
			SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_MEANING, WM_SETTEXT, 0, (LPARAM)str );

			// Check the radio button for 'Background (1)':
			SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_RADIO_BG1, BM_SETCHECK, BST_CHECKED, 0 );
			options_color_kanji_bg = color_kanji_bg1;

			return(TRUE);
		}
		case WM_COMMAND: {
			WORD identifier = LOWORD(wParam);
			WORD code = HIWORD(wParam);
			switch( identifier ) {
				case IDC_KANJIWINDOW_FONT_KANJI:
				case IDC_KANJIWINDOW_FONT_KANA:
				case IDC_KANJIWINDOW_FONT_MEANING: {
				/*		The 'Font' button for Kanji\Kana\Meaning was clicked.
					We need to pop up a choose-font box so he can choose the font.
					If a font was chosen ('ok'), then we need to update the registry
					and update the font that the kanji window uses.
				*/
					switch( code ) {
						case BN_CLICKED: {
							CHOOSEFONT cf;
								cf.lStructSize = sizeof(CHOOSEFONT);
								cf.hwndOwner = hWnd;
								cf.Flags = CF_FORCEFONTEXIST|CF_SCREENFONTS|CF_INITTOLOGFONTSTRUCT|CF_LIMITSIZE;
								static LOGFONT lf;
									switch( identifier ) {
										case IDC_KANJIWINDOW_FONT_KANJI: {
											wcscpy( lf.lfFaceName, ToWString( registry.GetSTRING(".\\Kanji\\Fonts\\Kanji\\Font") ).c_str() );
											lf.lfHeight = -MulDiv( registry.GetINT(".\\Kanji\\Fonts\\Kanji\\Size"), GetDeviceCaps(GetDC(hwnd_kanjiwindow), LOGPIXELSY), 72);
											break;
										}
										case IDC_KANJIWINDOW_FONT_ON: {
											wcscpy( lf.lfFaceName, ToWString( registry.GetSTRING(".\\Kanji\\Fonts\\On\\Font") ).c_str() );
											lf.lfHeight = -MulDiv( registry.GetINT(".\\Kanji\\Fonts\\On\\Size"), GetDeviceCaps(GetDC(hwnd_kanjiwindow), LOGPIXELSY), 72);
											break;
										}
										case IDC_KANJIWINDOW_FONT_MEANING: {
											wcscpy( lf.lfFaceName, ToWString( registry.GetSTRING(".\\Kanji\\Fonts\\Meaning\\Font") ).c_str() );
											lf.lfHeight = -MulDiv( registry.GetINT(".\\Kanji\\Fonts\\Meaning\\Size"), GetDeviceCaps(GetDC(hwnd_kanjiwindow), LOGPIXELSY), 72);
											break;
										}
									}
									lf.lfWeight = FW_NORMAL;
									lf.lfItalic = FALSE;
									lf.lfUnderline = FALSE;
									lf.lfStrikeOut = FALSE;
									lf.lfCharSet = SHIFTJIS_CHARSET;
								cf.lpLogFont = &lf;
								cf.nSizeMin = 6;
								cf.nSizeMax = 100;
							// Show the choose-font window:
							if( ChooseFont( &cf ) ) {
							// If the clicked 'ok'...
								switch( identifier ) {
									case IDC_KANJIWINDOW_FONT_KANJI: {
										// Kill the old font and make the new one:
										DeleteObject( hfont_kanji_kanji );
										hfont_kanji_kanji = CreateFontIndirect( &lf );

										// Set the new values:
										registry.SetSTRING(".\\Kanji\\Fonts\\Kanji\\Font", ToString(lf.lfFaceName).c_str() );
										registry.SetINT(".\\Kanji\\Fonts\\Kanji\\Size", (cf.iPointSize/10) );
								
										// Set the text of the box:
										Uint16 str[1024];
											swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Kanji\\Fonts\\Kanji\\Font"), registry.GetINT(".\\Kanji\\Fonts\\Kanji\\Size") );
										SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_KANJI, WM_SETTEXT, 0, (LPARAM)str );
										break;
									}
									case IDC_KANJIWINDOW_FONT_ON: {
										// Kill the old font and make the new one:
										DeleteObject( hfont_kanji_onkun );
										hfont_kanji_onkun = CreateFontIndirect( &lf );

										// Set the new values:
										registry.SetSTRING(".\\Kanji\\Fonts\\On\\Font", ToString(lf.lfFaceName).c_str() );
										registry.SetINT(".\\Kanji\\Fonts\\On\\Size", (cf.iPointSize/10) );
								
										// Set the text of the box:
										Uint16 str[1024];
											swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Kanji\\Fonts\\On\\Font"), registry.GetINT(".\\Kanji\\Fonts\\On\\Size") );
										SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_ON, WM_SETTEXT, 0, (LPARAM)str );
										SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_KUN, WM_SETTEXT, 0, (LPARAM)str );
										break;
									}
									case IDC_KANJIWINDOW_FONT_MEANING: {
										// Kill the old font and make the new one:
										DeleteObject( hfont_kanji_meaning );
										hfont_kanji_meaning = CreateFontIndirect( &lf );

										// Set the new values:
										registry.SetSTRING(".\\Kanji\\Fonts\\Meaning\\Font", ToString(lf.lfFaceName).c_str() );
										registry.SetINT(".\\Kanji\\Fonts\\Meaning\\Size", (cf.iPointSize/10) );
								
										// Set the text of the box:
										Uint16 str[1024];
											swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Kanji\\Fonts\\Meaning\\Font"), registry.GetINT(".\\Kanji\\Fonts\\Meaning\\Size") );
										SendDlgItemMessage( hWnd, IDC_KANJIWINDOW_MEANING, WM_SETTEXT, 0, (LPARAM)str );
										break;
									}
								}

								// Redraw the kanji window:
								// (We need to resize it to get the sizes right (new fonts).)
								RECT rc;
									GetWindowRect( hwnd_kanjiwindow, &rc );
								SendMessage( hwnd_kanjiwindow, WM_SIZE, SIZE_RESTORED, MAKELPARAM(rc.right-rc.left,rc.bottom-rc.top) );
								// Redraw it:
								RedrawWindow( hwnd_kanjiwindow, NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
				case IDC_KANJIWINDOW_COLOR_KANJI:
				case IDC_KANJIWINDOW_COLOR_ON:
				case IDC_KANJIWINDOW_COLOR_KUN:
				case IDC_KANJIWINDOW_COLOR_MEANING: {
				/*		The 'Color' button for kanji\kana\meaning was
					clicked.  Pick a new word color.
				*/
					switch( code ) {
						case BN_CLICKED: {
							// The custom-color array:
							static COLORREF colorref_array[16];
							// The choose-color structure:
							CHOOSECOLOR ch;
								ch.lStructSize = sizeof(CHOOSECOLOR);
								ch.hwndOwner = hWnd;
								switch( identifier ) {
									case IDC_KANJIWINDOW_COLOR_KANJI: {
										ch.rgbResult = color_kanji.ToCOLORREF();
										break;
									}
									case IDC_KANJIWINDOW_COLOR_ON: {
										ch.rgbResult = color_kanji_on.ToCOLORREF();
										break;
									}
									case IDC_KANJIWINDOW_COLOR_KUN: {
										ch.rgbResult = color_kanji_kun.ToCOLORREF();
										break;
									}
									case IDC_KANJIWINDOW_COLOR_MEANING: {
										ch.rgbResult = color_kanji_meaning.ToCOLORREF();
										break;
									}
								}
								ch.lpCustColors = colorref_array;
								ch.Flags = CC_RGBINIT|CC_FULLOPEN|CC_PREVENTFULLOPEN|CC_ANYCOLOR;
							// Open up the dialog box:
							if( ChooseColor(&ch) ) {
							// If he clicked on 'ok'...
								switch( identifier ) {
									case IDC_KANJIWINDOW_COLOR_KANJI: {
										// Set the color:
										color_kanji.SetFromCOLORREF(ch.rgbResult);
										registry.SetINT(".\\Kanji\\Fonts\\Kanji\\Color",ch.rgbResult );

										// Redraw the text area:
										RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_KANJI), NULL, NULL, RDW_INVALIDATE );
										break;
									}
									case IDC_KANJIWINDOW_COLOR_ON: {
										// Set the color:
										color_kanji_on.SetFromCOLORREF(ch.rgbResult);
										registry.SetINT(".\\Kanji\\Fonts\\On\\Color",ch.rgbResult );

										// Redraw the text area:
										RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_ON), NULL, NULL, RDW_INVALIDATE );
										break;
									}
									case IDC_KANJIWINDOW_COLOR_KUN: {
										// Set the color:
										color_kanji_kun.SetFromCOLORREF(ch.rgbResult);
										registry.SetINT(".\\Kanji\\Fonts\\Kun\\Color",ch.rgbResult );

										// Redraw the text area:
										RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_KUN), NULL, NULL, RDW_INVALIDATE );
										break;
									}
									case IDC_KANJIWINDOW_COLOR_MEANING: {
										// Set the color:
										color_kanji_meaning.SetFromCOLORREF(ch.rgbResult);
										registry.SetINT(".\\Kanji\\Fonts\\Meaning\\Color",ch.rgbResult );

										// Redraw the text area:
										RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_MEANING), NULL, NULL, RDW_INVALIDATE );
										break;
									}
								}
								// Redraw the results window:
								RedrawWindow( hwnd_kanjiwindow, NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
				case IDC_KANJIWINDOW_RADIO_BG1: {
				/*		When he clicks on 'Background (1)', set the background color
					of the edit boxes to the Kanji Window's color_bg.
				*/
					switch( HIWORD(wParam) ) {
						case BN_CLICKED: {
							if( IsDlgButtonChecked( hWnd, IDC_KANJIWINDOW_RADIO_BG1 ) ) {
								// Set the color:
								options_color_kanji_bg = color_kanji_bg1;
								// Delete and remake the brush:
								DeleteObject( hbrush_kanji );
								hbrush_kanji = CreateSolidBrush( options_color_kanji_bg.ToCOLORREF() );
								// Redraw the windows:
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_KANJI), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_ON), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_KUN), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_MEANING), NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
				case IDC_KANJIWINDOW_RADIO_BG2: {
				/*		When he clicks on 'Background (2)', set the background color
					of the edit boxes to the Kanji Window's color_bg2.
				*/
					switch( HIWORD(wParam) ) {
						case BN_CLICKED: {
							if( IsDlgButtonChecked( hWnd, IDC_KANJIWINDOW_RADIO_BG2 ) ) {
								// Set the color:
								options_color_kanji_bg = color_kanji_bg2;
								// Delete and remake the brush:
								DeleteObject( hbrush_kanji );
								hbrush_kanji = CreateSolidBrush( options_color_kanji_bg.ToCOLORREF() );
								// Redraw the windows:
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_KANJI), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_ON), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_KUN), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_MEANING), NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
				case IDC_KANJIWINDOW_COLOR_BG: {
				/*		When the background 'Color' button is clicked, pick a new background
					color for the currently selected background mode.
				*/
					switch( HIWORD(wParam) ) {
						case BN_CLICKED: {
							// Find out what button (1 or 2) is pressed:
							int mode = IsDlgButtonChecked( hWnd, IDC_KANJIWINDOW_RADIO_BG1 )?1:2;
							// The custom-color array:
							static COLORREF colorref_array[16];
							// The choose-color structure:
							CHOOSECOLOR ch;
								ch.lStructSize = sizeof(CHOOSECOLOR);
								ch.hwndOwner = hWnd;
								ch.rgbResult = options_color_kanji_bg.ToCOLORREF();
								ch.lpCustColors = colorref_array;
								ch.Flags = CC_RGBINIT|CC_FULLOPEN|CC_PREVENTFULLOPEN|CC_ANYCOLOR;
							// Open up the dialog box:
							if( ChooseColor(&ch) ) {
							// If he clicked on 'ok'...
								if( mode == 1 ) {									
									color_kanji_bg1.SetFromCOLORREF(ch.rgbResult);
									options_color_kanji_bg = color_kanji_bg1;
									registry.SetINT(".\\Kanji\\Colors\\BG1",ch.rgbResult );
								} else {
									color_kanji_bg2.SetFromCOLORREF(ch.rgbResult);
									options_color_kanji_bg = color_kanji_bg2;
									registry.SetINT(".\\Kanji\\Colors\\BG2",ch.rgbResult );
								}
								// Delete the brush and re-make it:
								DeleteObject( hbrush_kanji );
								hbrush_kanji = CreateSolidBrush( options_color_kanji_bg.ToCOLORREF() );
								// Redraw the text areas:
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_KANJI), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_ON), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_KUN), NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( GetDlgItem(hWnd,IDC_KANJIWINDOW_MEANING), NULL, NULL, RDW_INVALIDATE );
								// Redraw the results window:
								RedrawWindow( hwnd_kanjiwindow, NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
				case IDC_KANJIWINDOW_COLOR_WINDOW:
				case IDC_KANJIWINDOW_COLOR_BORDER_OUTSIDE:
				case IDC_KANJIWINDOW_COLOR_BORDER_INSIDE: {
				/*		The 'Color' button for the:
					* inside border
					* outside border
					* window
					was clicked.  Pick a new color.
				*/
					switch( code ) {
						case BN_CLICKED: {
							// The custom-color array:
							static COLORREF colorref_array[16];
							// The choose-color structure:
							CHOOSECOLOR ch;
								ch.lStructSize = sizeof(CHOOSECOLOR);
								ch.hwndOwner = hWnd;
								switch( identifier ) {
									case IDC_KANJIWINDOW_COLOR_BORDER_OUTSIDE: {
										ch.rgbResult = color_kanji_border_outside.ToCOLORREF();
										break;
									}
									case IDC_KANJIWINDOW_COLOR_BORDER_INSIDE: {
										ch.rgbResult = color_kanji_border_inside.ToCOLORREF();
										break;
									}
									case IDC_KANJIWINDOW_COLOR_WINDOW: {
										ch.rgbResult = color_kanji_windowbg.ToCOLORREF();
										break;
									}
								}
								ch.lpCustColors = colorref_array;
								ch.Flags = CC_RGBINIT|CC_FULLOPEN|CC_PREVENTFULLOPEN|CC_ANYCOLOR;
							// Open up the dialog box:
							if( ChooseColor(&ch) ) {
							// If he clicked on 'ok'...
								switch( identifier ) {
									case IDC_KANJIWINDOW_COLOR_BORDER_OUTSIDE: {
										// Set the color:
										color_kanji_border_outside.SetFromCOLORREF(ch.rgbResult);
										registry.SetINT(".\\Kanji\\Colors\\Border_Outside",ch.rgbResult );
										break;
									}
									case IDC_KANJIWINDOW_COLOR_BORDER_INSIDE: {
										// Set the color:
										color_kanji_border_inside.SetFromCOLORREF(ch.rgbResult);
										registry.SetINT(".\\Kanji\\Colors\\Border_Inside",ch.rgbResult );
										break;
									}
									case IDC_KANJIWINDOW_COLOR_WINDOW: {
										// Set the color:
										color_kanji_windowbg.SetFromCOLORREF(ch.rgbResult);
										registry.SetINT(".\\Kanji\\Colors\\BGWindow",ch.rgbResult );
										break;
									}
								}
								// Redraw the results window:
								RedrawWindow( hwnd_kanjiwindow, NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
			}
			return(TRUE);
		}
		case WM_CTLCOLORSTATIC: {
		/*		This message is sent when an edit box needs to be drawn.
			It gives you the ability to set the foreground and background colors.
			Also, you return the brush that you want use to color the rest
			(non-text) areas.
		*/
			HDC hdc_edit = (HDC)wParam;
			HWND hwnd_edit = (HWND)lParam;

			switch( GetDlgCtrlID(hwnd_edit) ) {
				case IDC_KANJIWINDOW_KANJI: {
					SetTextColor( hdc_edit, color_kanji.ToCOLORREF() );
					SetBkColor( hdc_edit, options_color_kanji_bg.ToCOLORREF() );
					return( (BOOL)hbrush_kanji );
				}
				case IDC_KANJIWINDOW_ON: {
					SetTextColor( hdc_edit, color_kanji_on.ToCOLORREF() );
					SetBkColor( hdc_edit, options_color_kanji_bg.ToCOLORREF() );
					return( (BOOL)hbrush_kanji );
				}
				case IDC_KANJIWINDOW_KUN: {
					SetTextColor( hdc_edit, color_kanji_kun.ToCOLORREF() );
					SetBkColor( hdc_edit, options_color_kanji_bg.ToCOLORREF() );
					return( (BOOL)hbrush_kanji );
				}
				case IDC_KANJIWINDOW_MEANING: {
					SetTextColor( hdc_edit, color_kanji_meaning.ToCOLORREF() );
					SetBkColor( hdc_edit, options_color_kanji_bg.ToCOLORREF() );
					return( (BOOL)hbrush_kanji );
				}
			}
			return(FALSE);
		}
	}
	// Returning FALSE causes it to do the default processing for the message.
   return(FALSE); 
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateDialog_Options_KanjiWindow( HWND hwndParent ) {
	HWND hwnd = CreateDialog(
		g_hinst,
		MAKEINTRESOURCE( IDD_OPTIONS_KANJIWINDOW ), 
		hwndParent,
		(DLGPROC)DlgProc_Options_KanjiWindow
	);
	return( hwnd );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
