// Dialog_Options_SearchWindow.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef DIALOG_OPTIONS_SEARCHWINDOW_H
#define DIALOG_OPTIONS_SEARCHWINDOW_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HBRUSH hbrush_search = CreateSolidBrush( 0 );
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
BOOL CALLBACK DlgProc_Options_SearchWindow( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {
	switch( message ) {
		case WM_INITDIALOG: {
		/* wParam
				Handle to the control to receive the default keyboard focus. The system assigns the default keyboard focus only if the dialog box procedure returns TRUE. 
			lParam
				Specifies additional initialization data. This data is passed to the system as the lParam parameter in a call to the CreateDialogIndirectParam, CreateDialogParam, DialogBoxIndirectParam, or DialogBoxParam function used to create the dialog box. For property sheets, this parameter is a pointer to the PROPSHEETPAGE structure used to create the page. This parameter is zero if any other dialog box creation function is used. 
			Return: TRUE if we want to set keyboard focus to wParam, FALSE otherwise.
		*/

			// Set the text of the font boxes:
			Uint16 str[1024];
				swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Search\\Font\\Font"), registry.GetINT(".\\Search\\Font\\Size") );
			SendDlgItemMessage( hWnd, IDC_SEARCHWINDOW_SEARCHTEXT, WM_SETTEXT, 0, (LPARAM)str );

			// Delete the brush and re-make it:
			if( hbrush_search ) {
				DeleteObject( hbrush_search );
			}
			hbrush_search = CreateSolidBrush( registry.GetINT(".\\Search\\WindowBG") );

			return(TRUE);
		}
		case WM_COMMAND: {
			WORD identifier = LOWORD(wParam);
			WORD code = HIWORD(wParam);
			switch( identifier ) {
				case IDC_SEARCHWINDOW_FONT_SEARCHTEXT: {
				/*		The 'Font' button for searchtext was clicked.
					We need to pop up a choose-font box so he can choose the font.
					If a font was chosen ('ok'), then we need to update the registry
					and update the font that the result window uses.
				*/
					switch( code ) {
						case BN_CLICKED: {
							CHOOSEFONT cf;
								cf.lStructSize = sizeof(CHOOSEFONT);
								cf.hwndOwner = hWnd;
								cf.Flags = CF_FORCEFONTEXIST|CF_SCREENFONTS|CF_INITTOLOGFONTSTRUCT|CF_LIMITSIZE;
								static LOGFONT lf;
									switch( identifier ) {
										case IDC_SEARCHWINDOW_FONT_SEARCHTEXT: {
											wcscpy( lf.lfFaceName, ToWString( registry.GetSTRING(".\\Search\\Font\\Font") ).c_str() );
											lf.lfHeight = -MulDiv( registry.GetINT(".\\Search\\Font\\Size"), GetDeviceCaps(GetDC(hwnd_searchwindow), LOGPIXELSY), 72);
											break;
										}
									}
									lf.lfWeight = FW_NORMAL;
									lf.lfItalic = FALSE;
									lf.lfUnderline = FALSE;
									lf.lfStrikeOut = FALSE;
									lf.lfCharSet = SHIFTJIS_CHARSET;
								cf.lpLogFont = &lf;
								cf.nSizeMin = 6;
								cf.nSizeMax = 100;
							// Show the choose-font window:
							if( ChooseFont( &cf ) ) {
							// If the clicked 'ok'...
								switch( identifier ) {
									case IDC_SEARCHWINDOW_FONT_SEARCHTEXT: {
										// Set the new values:
										registry.SetSTRING(".\\Search\\Font\\Font", ToString(lf.lfFaceName).c_str() );
										registry.SetINT(".\\Search\\Font\\Size", (cf.iPointSize/10) );
								
										// Set the text of the box:
										Uint16 str[1024];
											swprintf(str,L"%S, %i pt.",registry.GetSTRING(".\\Search\\Font\\Font"), registry.GetINT(".\\Search\\Font\\Size") );
										SendDlgItemMessage( hWnd, IDC_SEARCHWINDOW_SEARCHTEXT, WM_SETTEXT, 0, (LPARAM)str );

										CHARFORMAT2 fmt;
											fmt.cbSize = sizeof(CHARFORMAT2);
											fmt.dwMask = CFM_FACE;
											wcscpy( fmt.szFaceName, lf.lfFaceName );
										SendMessage( hwnd_searchedit, EM_SETCHARFORMAT, SCF_ALL, (LPARAM)&fmt );
										break;
									}
								}

								// Redraw the search window:
								RedrawWindow( hwnd_searchwindow, NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
				case IDC_SEARCHWINDOW_COLOR_SEARCHTEXT: {
				/*		The 'Color' button for word\kana\meaning\info was
					clicked.  Pick a new word color.
				*/
					switch( code ) {
						case BN_CLICKED: {
							// The custom-color array:
							static COLORREF colorref_array[16];
							// The choose-color structure:
							CHOOSECOLOR ch;
								ch.lStructSize = sizeof(CHOOSECOLOR);
								ch.hwndOwner = hWnd;
								switch( identifier ) {
									case IDC_RESULTWINDOW_COLOR_WORD: {
										ch.rgbResult = registry.GetINT(".\\Search\\Font\\Color");
										break;
									}
								}
								ch.lpCustColors = colorref_array;
								ch.Flags = CC_RGBINIT|CC_FULLOPEN|CC_PREVENTFULLOPEN|CC_ANYCOLOR;
							// Open up the dialog box:
							if( ChooseColor(&ch) ) {
							// If he clicked on 'ok'...
								switch( identifier ) {
									case IDC_SEARCHWINDOW_COLOR_SEARCHTEXT: {
										registry.SetINT(".\\Search\\Font\\Color",ch.rgbResult );

										CHARFORMAT2 fmt;
											fmt.cbSize = sizeof(CHARFORMAT2);
											fmt.dwMask = CFM_COLOR;
											fmt.crTextColor = ch.rgbResult;
										SendMessage( hwnd_searchedit, EM_SETCHARFORMAT, SCF_ALL, (LPARAM)&fmt );

										// Redraw the text area:
										RedrawWindow( GetDlgItem(hWnd,IDC_SEARCHWINDOW_SEARCHTEXT), NULL, NULL, RDW_INVALIDATE );
										break;
									}
								}
								// Redraw the results window:
								RedrawWindow( hwnd_searchwindow, NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
				case IDC_SEARCHWINDOW_COLOR_WINDOW: {
				/*		The 'Color' button for the:
					* window
					was clicked.  Pick a new color.
				*/
					switch( code ) {
						case BN_CLICKED: {
							// The custom-color array:
							static COLORREF colorref_array[16];
							// The choose-color structure:
							CHOOSECOLOR ch;
								ch.lStructSize = sizeof(CHOOSECOLOR);
								ch.hwndOwner = hWnd;
								switch( identifier ) {
									case IDC_SEARCHWINDOW_COLOR_WINDOW: {
										ch.rgbResult = registry.GetINT(".\\Search\\WindowBG");
										break;
									}
								}
								ch.lpCustColors = colorref_array;
								ch.Flags = CC_RGBINIT|CC_FULLOPEN|CC_PREVENTFULLOPEN|CC_ANYCOLOR;
							// Open up the dialog box:
							if( ChooseColor(&ch) ) {
							// If he clicked on 'ok'...
								switch( identifier ) {
									case IDC_SEARCHWINDOW_COLOR_WINDOW: {
										// Set the color:
										registry.SetINT(".\\Search\\WindowBG",ch.rgbResult );

										// Delete the brush and re-make it:
										DeleteObject( hbrush_search );
										hbrush_search = CreateSolidBrush( ch.rgbResult );

										//SetClassLong( hwnd_searchwindow, GCL_HBRBACKGROUND, (LONG)hbrush_search );
										SendMessage( hwnd_searchedit, EM_SETBKGNDCOLOR, FALSE, ch.rgbResult );

										RedrawWindow( GetDlgItem(hWnd,IDC_SEARCHWINDOW_SEARCHTEXT), NULL, NULL, RDW_INVALIDATE );
										break;
									}
								}
								// Redraw the search window:
								RedrawWindow( hwnd_searchwindow, NULL, NULL, RDW_INVALIDATE );
							}
							break;
						}
					}
					break;
				}
			}
			return(TRUE);
		}
		case WM_CTLCOLORSTATIC: {
		/*		This message is sent when an edit box needs to be drawn.
			It gives you the ability to set the foreground and background colors.
			Also, you return the brush that you want use to color the rest
			(non-text) areas.
		*/
			HDC hdc_edit = (HDC)wParam;
			HWND hwnd_edit = (HWND)lParam;

			switch( GetDlgCtrlID(hwnd_edit) ) {
				case IDC_SEARCHWINDOW_SEARCHTEXT: {
					SetTextColor( hdc_edit, registry.GetINT(".\\Search\\Font\\Color") );
					SetBkColor( hdc_edit, registry.GetINT(".\\Search\\WindowBG") );
					return( (BOOL)hbrush_search );
				}
			}
			return(FALSE);
		}
	}
	// Returning FALSE causes it to do the default processing for the message.
   return(FALSE); 
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateDialog_Options_SearchWindow( HWND hwndParent ) {
	HWND hwnd = CreateDialog(
		g_hinst,
		MAKEINTRESOURCE( IDD_OPTIONS_SEARCHWINDOW ), 
		hwndParent,
		(DLGPROC)DlgProc_Options_SearchWindow
	);
	return( hwnd );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
