// EDICT_Sort.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef EDICT_SORT_H
#define EDICT_SORT_H
///////////////////////////////////////////////////////////
#include <Algorithm>
///////////////////////////////////////////////////////////
/* This is the SORT_BY structure.
   It's used by the sort functions to know how to actually
	sort everything.
   * window_id is used to know what set of attributes to use.
   * index is the index in the header that is window_id.
   * sortstyle is ASCENDING or DESCENDING. */
struct SORT_BY {
	int window_id, index;
	int sortstyle;
};

/* SortStyle:
   This is either ASCENDING or DESCENDING. */
enum {	ASCENDING = 0,
			DESCENDING = 1	};

/* This is the enumeration of PRIMARY and SECONDARY, for 
   use with the sort_by array. */
enum {	PRIMARY = 0,
			SECONDARY	};

/* sort_by[] keeps information on how to actually do the
   sort.  See the SORT_BY structure. */
SORT_BY sort_by[2] = { {0,0},{0,0} };
/* sort_on is used by the sort functions to pick what
   attribute to sort on.
   Those functions will use sort_by[sort_on] to do it. */
int sort_on = PRIMARY;

/* This determines if two entries are equal. */
bool operator==( const EDICT_Entry &a, const EDICT_Entry &b ) {
	switch( sort_by[sort_on].window_id ) {
		case WID_RESULT_LIST1: {
			switch( sort_by[sort_on].index ) {
				case 0: { // WORD
					return( a.word == b.word );
					break;
				}
				case 1: { // TYPE
					break;
				}
				case 2: { // LENGTH: WORD
					return( a.word.size() == b.word.size() );
					break;
				}
				case 3: { // LENGTH: KANA
					return( a.kana.size() == b.kana.size() );
					break;
				}
			}
			break;
		}
		case WID_RESULT_LIST2: {
			switch( sort_by[sort_on].index ) {
				case 0: { // KANA
					return( a.kana == b.kana );
					break;
				}
				case 1: { // MEANING
					if( a.meaning.size() >= 1 && b.meaning.size() >= 1 ) {
						return( a.meaning[0][0] == b.meaning[0][0] );
					} else {
						fwprintf(g_flogu,L"Sort: ==: ERROR: a.meaning.size or b.meaning.size < 1");
					}
					break;
				}
			}
			break;
		}
	}//*/
	return( a.word.size() == b.word.size() );
}

/* This uses sort_by[sort_on] to determine if one entry
   is 'greater' than another. */
bool operator>( const EDICT_Entry &a, const EDICT_Entry &b ) {
	switch( sort_by[sort_on].window_id ) {
		case WID_RESULT_LIST1: {
			switch( sort_by[sort_on].index ) {
				case 0: { // WORD
					if( sort_by[sort_on].sortstyle == ASCENDING ) {
						return( a.word > b.word );
					} else {
						return( !(a.word > b.word) );
					}
					break;
				}
				case 1: { // TYPE
					break;
				}
				case 2: { // LENGTH: WORD
					if( sort_by[sort_on].sortstyle == ASCENDING ) {
						return( a.word.size() > b.word.size() );
					} else {
						return( !(a.word.size() > b.word.size()) );
					}
					break;
				}
				case 3: { // LENGTH: KANA
					if( sort_by[sort_on].sortstyle == ASCENDING ) {
						return( a.kana.size() > b.kana.size() );
					} else {
						return( !(a.kana.size() > b.kana.size()) );
					}
					break;
				}
			}
			break;
		}
		case WID_RESULT_LIST2: {
			switch( sort_by[sort_on].index ) {
				case 0: { // KANA
					if( sort_by[sort_on].sortstyle == ASCENDING ) {
						return( a.kana > b.kana );
					} else {
						return( !(a.kana > b.kana) );
					}
					break;
				}
				case 1: { // MEANING
					if( a.meaning.size() >= 1 && b.meaning.size() >= 1 ) {
						if( sort_by[sort_on].sortstyle == ASCENDING ) {
							return( a.meaning[0][0] > b.meaning[0][0] );
						} else {
							return( !(a.meaning[0][0] > b.meaning[0][0]) );
						}
					} else {
						fwprintf(g_flogu,L"Sort: ==: ERROR: a.meaning.size or b.meaning.size < 1");
					}
					break;
				}
			}
			break;
		}
	}//*/
	return( a.word.size()>b.word.size() );
}

/* If it's not greater than an entry and it's not equal
   to it, then it must be less than it. */
bool operator<( const EDICT_Entry &a, const EDICT_Entry &b ) {
	return( !( a > b ) && !(a == b) );
}

/* Sort the results by the primary and secondary sorts.
   If there is no secondary, it skips it.
	If there is no primary, it skips it.
//*/
void SortResults() {
	if( edict_results.size() > 0 ) {
		// If there is a secondary way to sort, do it:
		if( sort_by[SECONDARY].window_id != 0 ) {
			sort_on = SECONDARY;
				sort( edict_results.begin(), edict_results.end() );
		}
		// Sort by the primary:
		if( sort_by[PRIMARY].window_id != 0 ) {
			sort_on = PRIMARY;
				sort( edict_results.begin(), edict_results.end() );
			RedrawWindow( hwnd_resultwindow, NULL, NULL, RDW_INVALIDATE );
		}
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
