// Kanji_Window.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef KANJI_WINDOW_H
#define KANJI_WINDOW_H
///////////////////////////////////////////////////////////
// For its header bars:
#include "Kanji_List.h"
#include "KANJIDIC_Sort.h"

// For copying information:
#include "Kanji_Copy.h"

// For inverting colors:
#include "Invert.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// For drawing the entries:
//int bx = 3, by = 3;
int kanji_height = 0;
	int kanji_lineh1 = 0;
	int kanji_lineh2 = 0;

// Where to start drawing kanji entries:
int start_kanji = 0;

// This is for maintaining the size of the client area:
RECT rcClient_kanji;

// Colors:
Color color_kanji( 0,0,96 );
Color color_kanji_on( 96,0,0 );
Color color_kanji_kun( 0,96,0 );
Color color_kanji_meaning( 0,0,0 );

Color color_kanji_bg1( 255,255,255 );
Color color_kanji_bg2( 196,196,196 );

Color color_kanji_border_inside( 128,128,128 );
Color color_kanji_border_outside( 0,0,0 );

// This is the background color of the whole window:
Color color_kanji_windowbg( 255,255,255 );

// Fonts to use:
HFONT hfont_kanji_kanji = CreateFont(30, 0,0,0,0,0,0,0,0,0,0,0,0,L"MS Mincho" );
HFONT hfont_kanji_onkun = CreateFont(12, 0,0,0,0,0,0,0,0,0,0,0,0,L"MS Mincho" );
HFONT hfont_kanji_meaning = CreateFont(14, 0,0,0,0,0,0,0,0,0,0,0,0,L"Arial" );
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// For the tooltip-like boxes:
#include "Kanji_Hover.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
int KanjiWindow_GetEntryFromPoint( int mx, int my ) {
	// Get vertical scroll bar position
	SCROLLINFO si;
	   si.cbSize = sizeof(si);
	   si.fMask  = SIF_POS;
	GetScrollInfo( hwnd_kanjiwindow, SB_VERT, &si );
	start_kanji = si.nPos;

				
	int entries_that_fit_on_screen = (rcClient_kanji.bottom-rcClient_kanji.top)/(kanji_height);
	int end_kanji = start_kanji + min( entries_that_fit_on_screen, kanjidic_results.size()-1-start_kanji );

	int return_kanji = -1;
	for( int e = start_kanji; e <= end_kanji; e++ ) {
		int i = e-start_kanji;

		if( my >= rcClient_kanji.top + i*kanji_height &&
			 my <= rcClient_kanji.top + (i+1)*kanji_height-1 ) {
			return_kanji = e;
			break;
		}
	}

	return(return_kanji);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc_Kanji_Window(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	// For the tooltip-like boxes:
	static POINT hover_pos;
	static int hover_entry = -1;
	static bool hover_on = false;
	static HWND hwnd_hover = NULL;

	switch( message ) {
		case WM_CREATE: {
			hwnd_kanjilist1 = CreateKanji_List1( hWnd );
			RECT rcList1;
				GetClientRect( hwnd_kanjilist1, &rcList1 );
			hwnd_kanjilist2 = CreateKanji_List2( hWnd, 0,rcList1.bottom );//*/
			return(0);
		}
		case WM_SIZE: {
			Graphics graphics( GetDC(hWnd) );
				kanji_lineh1 = by + max(
					Font( GetDC(hWnd),hfont_kanji_onkun ).GetHeight( &graphics ),
					Font( GetDC(hWnd),hfont_kanji_meaning ).GetHeight( &graphics )
					) + by;
				kanji_lineh2 = by + max(
					Font( GetDC(hWnd),hfont_kanji_meaning ).GetHeight( &graphics ),
					0
					) + by;
			kanji_height = kanji_lineh1 + kanji_lineh2;

			// Fix the windows:
			RECT rcWindow;
				GetClientRect( hWnd, &rcWindow );

			RECT rcList1;
				GetClientRect( hwnd_kanjilist1, &rcList1 );		
			RECT rcList2;
				GetClientRect( hwnd_kanjilist2, &rcList2 );
			if( rcWindow.right > 100 ) {
				Header_SetSizeScale( hwnd_kanjilist1, rcWindow.right,rcList1.bottom );
				Header_SetSizeScale( hwnd_kanjilist2, rcWindow.right,rcList2.bottom );
			}


			// Retrieve the dimensions of the client area. 
			int xClient = LOWORD(lParam);
			int yClient = HIWORD(lParam);

			// Set the client information:
			rcClient_kanji.left = 0;
			rcClient_kanji.top = 0 + rcList1.bottom + rcList1.bottom;

			rcClient_kanji.right = xClient;
			rcClient_kanji.bottom = yClient;

			
			// Set the vertical scrolling range and page size
			SCROLLINFO si;
				si.cbSize = sizeof(si); 
				si.fMask  = SIF_RANGE | SIF_PAGE;// | SIF_DISABLENOSCROLL; 
				si.nMin   = 0;
				si.nMax   = kanjidic_results.size() - 1;
				si.nPage  = (rcClient_kanji.bottom-rcClient_kanji.top) / (kanji_height); 
			SetScrollInfo(hWnd, SB_VERT, &si, TRUE);//*/
			
			// Recalculate the start based on the new size:
				si.fMask  = SIF_ALL;
         GetScrollInfo( hWnd, SB_VERT, &si );
				start_kanji = si.nPos;

			// This makes resizing smooth:
			SendMessage( hWnd, WM_PAINT, 0,0 );
			return( 0 );
		}
		case WM_VSCROLL: {
         // Get all the vertial scroll bar information
			SCROLLINFO si; 
				si.cbSize = sizeof(si);
				si.fMask  = SIF_ALL;
         GetScrollInfo (hWnd, SB_VERT, &si);
         // Save the position for comparison later on
         start_kanji = si.nPos;
         switch( LOWORD(wParam) ) {
				// user clicked the HOME keyboard key
				case SB_TOP: {  si.nPos = si.nMin;  break;  }  
				// user clicked the END keyboard key
				case SB_BOTTOM: {  si.nPos = si.nMax;  break;  }
				// user clicked the top arrow
				case SB_LINEUP: {  si.nPos -= 1;  break;  }
				// user clicked the bottom arrow
				case SB_LINEDOWN: {  si.nPos += 1;  break;  }  
				// user clicked the shaft above the scroll box
				case SB_PAGEUP: {  si.nPos -= si.nPage;  break;  }  
				// user clicked the shaft below the scroll box
				case SB_PAGEDOWN: {  si.nPos += si.nPage;  break;  }  
				// user dragged the scroll box
				case SB_THUMBTRACK: {  si.nPos = si.nTrackPos;  break;  }
         }
         // Set the position and then retrieve it.  Due to adjustments
         //   by Windows it may not be the same as the value set.
				si.fMask = SIF_POS;
         SetScrollInfo( hWnd, SB_VERT, &si, TRUE );
         GetScrollInfo( hWnd, SB_VERT, &si );
         // If the position has changed, scroll window and update it
         if( si.nPos != start_kanji ) {
				RECT rc;
					GetClientRect( hWnd,&rc );
				RedrawWindow( hWnd, &rc, NULL, RDW_INVALIDATE );
				// RDW_ERASE makes it redraw the BG.
				// RDW_INVALIDATE makes it WM_PAINT.
         }
			return(0);
		}
		case WM_PAINT: {
			if( hwnd_kanjilist1 == NULL || hwnd_kanjilist2 == NULL ) {  break;  }

			// If there are no results, then just clear the thing:
			if( kanjidic_results.size() < 1 ) {
				HDC hdc;
				PAINTSTRUCT ps;
				hdc = BeginPaint(hWnd, &ps);
				   Graphics graphics(hdc);

					SolidBrush brush_white( Color(255,255,255) );

					RectF rcFill;
						rcFill.X = rcClient_kanji.left;
						rcFill.Y = rcClient_kanji.top;
						rcFill.Width = rcClient_kanji.right;
						rcFill.Height = rcClient_kanji.bottom;
					graphics.FillRectangle( &brush_white, rcFill );

				EndPaint(hWnd, &ps);
				return(0);
			}
		
			HDC hdc;
			PAINTSTRUCT ps;
	      hdc = BeginPaint(hWnd, &ps);
				Graphics graphics(hdc);

				Font font_kanji( hdc,hfont_kanji_kanji );
				Font font_onkun( hdc,hfont_kanji_onkun );
				Font font_meaning( hdc,hfont_kanji_meaning );

				SolidBrush brush_kanji_windowbg( color_kanji_windowbg );

				StringFormat format(StringFormatFlagsNoWrap);
					format.SetTrimming( StringTrimmingNone );
				StringFormat format_kanji(StringFormatFlagsNoWrap);
					format_kanji.SetTrimming( StringTrimmingNone );
					format_kanji.SetAlignment( StringAlignmentCenter );




				RECT rcKanji;
					Header_GetItemRect( hwnd_kanjilist1, 0, &rcKanji );
				RECT rcOn;
					Header_GetItemRect( hwnd_kanjilist1, 1, &rcOn );
				RECT rcKun;
					Header_GetItemRect( hwnd_kanjilist1, 2, &rcKun );
				RECT rcStrokes;
					Header_GetItemRect( hwnd_kanjilist1, 3, &rcStrokes );

				RECT rcKanji2;
					Header_GetItemRect( hwnd_kanjilist2, 0, &rcKanji2 );
				RECT rcMeaning;
					Header_GetItemRect( hwnd_kanjilist2, 1, &rcMeaning );


				// Get vertical scroll bar position
				SCROLLINFO si;
				   si.cbSize = sizeof(si);
		         si.fMask  = SIF_POS;
		      GetScrollInfo( hWnd, SB_VERT, &si );
			   start_kanji = si.nPos;//*/
				
				int entries_that_fit_on_screen = (rcClient_kanji.bottom-rcClient_kanji.top)/(kanji_height);
				int end_kanji = start_kanji + min( entries_that_fit_on_screen, kanjidic_results.size()-1-start_kanji );
				
				//fprintf(g_flog,"kanji: start_kanji: %i | end_kanji: %i\n",start_kanji, end_kanji );

				for( int e = start_kanji; e <= end_kanji; e++ ) {
					int i = e-start_kanji;

					KANJIDIC_Entry entry = kanjidic_results[e];


					SolidBrush brush_bg( color_kanji_bg1 );
					if( e%2 ) {	brush_bg.SetColor( color_kanji_bg2 );	}
					SolidBrush brush_kanji( color_kanji );
					SolidBrush brush_on( color_kanji_on );
					SolidBrush brush_kun( color_kanji_kun );
					SolidBrush brush_meaning( color_kanji_meaning );
					Pen pen_border_inside( color_kanji_border_inside );
					Pen pen_border_outside( color_kanji_border_outside );
					if( kanji_selected == e ) {
						InvertColor( brush_bg );
						InvertColor( brush_kanji );
						InvertColor( brush_on );
						InvertColor( brush_kun );
						InvertColor( brush_meaning );
						InvertColor( pen_border_inside );
						InvertColor( pen_border_outside );
					}



					RectF rc;
						rc.X = 0;
						rc.Y = rcClient_kanji.top + i*kanji_height;
						rc.Width = rcClient_kanji.right-1;
						rc.Height = kanji_height-1;
					graphics.FillRectangle( &brush_bg, rc );
					

					// KANJI:
					// Box:
						rc.X = rcKanji.left;
						rc.Y = rcClient_kanji.top + i*kanji_height;
						rc.Width = rcKanji.right - rcKanji.left - 1;
						rc.Height = kanji_lineh1+kanji_lineh2 - 1;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					Uint16 strkanji[2] = {  entry.kanji, L'\0'  };
					graphics.DrawString( strkanji, -1, &font_kanji, rc, &format_kanji, &brush_kanji );

					// ON:
					wstring on = Kanji_GetOn( entry );
					// Box:
						rc.X = rcOn.left;
						rc.Y = rcClient_kanji.top + i * kanji_height;
						rc.Width = rcOn.right - rcOn.left - 1;
						rc.Height = kanji_lineh1;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					graphics.DrawString( on.c_str(), -1, &font_onkun, rc, &format, &brush_on );

					// KUN:
					wstring kun = Kanji_GetKun( entry );
					// Box:
						rc.X = rcKun.left;
						rc.Y = rcClient_kanji.top + i*kanji_height;
						rc.Width = rcKun.right - rcKun.left - 1;
						rc.Height = kanji_lineh1;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					graphics.DrawString( kun.c_str(), -1, &font_onkun, rc, &format, &brush_kun );

					// STROKE COUNT:
					// Box:
						rc.X = rcStrokes.left;
						rc.Y = rcClient_kanji.top + i*kanji_height;
						rc.Width = rcStrokes.right - rcStrokes.left - 1;
						rc.Height = kanji_lineh1;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					if( entry.strokecount > 0 ) {
						Uint16 strstrokes[10]; swprintf(strstrokes,L"%i",entry.strokecount );
						graphics.DrawString( strstrokes, -1, &font_meaning, rc, &format, &brush_meaning );
					} else {
						//fprintf(g_flog,"WARNING: Kanji U%X has no stroke count.\n",entry.kanji );
					}


					// KANJI:
					/*// Box:
						rc.X = rcKana.left;
						rc.Y = rcClient_kanji.top + i*kanji_height + lineh1;
						rc.Width = rcKana.right - rcKana.left - 1;
						rc.Height = lineh2;
					graphics.DrawRectangle( &pen, rc );
					// Text:
						rc.X += bx;
						rc.Width -= bx;
						rc.Y += by;
						rc.Height -= by;
					graphics.DrawString( entry.kana.c_str(), -1, &font_kana, rc, &format, &brush_kana );//*/

					// MEANING:
					wstring meaning = Kanji_GetMeaning( entry );
					// Box:
						rc.X = rcMeaning.left;
						rc.Y = rcClient_kanji.top + i * kanji_height + kanji_lineh1;
						rc.Width = rcMeaning.right - rcMeaning.left - 1;
						rc.Height = kanji_lineh2;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					graphics.DrawString( meaning.c_str(), -1, &font_meaning, rc, &format, &brush_meaning );
					//*/


					// Draw a black bounding rectangle around everything:
						rc.X = 0;
						rc.Y = rcClient_kanji.top + i * kanji_height;
						rc.Width = rcClient_kanji.right-1;
						rc.Height = kanji_height-1;
					graphics.DrawRectangle( &pen_border_outside, rc );//*/
				}//*/


				// Fill in the remaining space:
				SolidBrush brush_bg( color_kanji_windowbg );
				RectF rcFill;
					rcFill.X = rcClient_kanji.left;
					rcFill.Y = rcClient_kanji.top + (end_kanji-start_kanji+1)*kanji_height;
					rcFill.Width = rcClient_kanji.right;
					rcFill.Height = rcClient_kanji.top + rcClient_kanji.bottom - (int)rcFill.Y;
				if( rcFill.Height > 0 ) {
					graphics.FillRectangle( &brush_kanji_windowbg, rcFill );
				}


		   EndPaint(hWnd, &ps);
			return(0);
		}//*/
		case WM_ERASEBKGND: {
			return(TRUE);
		}
		case WM_NOTIFY: {
			if( wParam == WID_KANJI_LIST1 || wParam == WID_KANJI_LIST2 ) {
				NMHEADER *header = (NMHEADER*)lParam;
				switch( header->hdr.code ) {
					case HDN_ITEMCLICK: {
					/* When an item is clicked, sort by that item.
					   The old PRIMARY becomes the new SECONDARY. */

						// Do not allow sorting by 'Kanji' (the bottom one):
						if( wParam == WID_KANJI_LIST2 && header->iItem == 0 ) {
							break;
						}

						// If he clicked the same item twice in a row:
						if( sort_kanji_by[PRIMARY].window_id == wParam && sort_kanji_by[PRIMARY].index == header->iItem ) {
							// Toggle it so that it sorts the other way:
							sort_kanji_by[PRIMARY].sortstyle ^= 1;

							// Give it an icon: (primary)
							HDITEM item;
								item.mask = HDI_BITMAP | HDI_FORMAT;
							Header_GetItem( header->hdr.hwndFrom, header->iItem, &item );
								item.mask = HDI_BITMAP | HDI_FORMAT;
								item.hbm = (sort_kanji_by[PRIMARY].sortstyle==ASCENDING)?bitmap_sort_asc1:bitmap_sort_dsc1;
								item.fmt |= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
							Header_SetItem( header->hdr.hwndFrom, header->iItem, &item );
						} else {
							// Take away the sort arrow on the oldest one:
							if( sort_kanji_by[SECONDARY].window_id != 0 ) {
								HDITEM item;
									item.mask = HDI_BITMAP | HDI_FORMAT;
								HWND hwndold = sort_kanji_by[SECONDARY].window_id==WID_KANJI_LIST1?hwnd_kanjilist1:hwnd_kanjilist2;
								Header_GetItem( hwndold, sort_kanji_by[SECONDARY].index, &item );
									item.mask = HDI_BITMAP | HDI_FORMAT;
									item.hbm = NULL;
									item.fmt ^= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
								Header_SetItem( hwndold, sort_kanji_by[SECONDARY].index, &item );
							}

							// The secondary sort is the old primary sort:
							sort_kanji_by[SECONDARY].window_id = sort_kanji_by[PRIMARY].window_id;
							sort_kanji_by[SECONDARY].index = sort_kanji_by[PRIMARY].index;
							sort_kanji_by[SECONDARY].sortstyle = sort_kanji_by[PRIMARY].sortstyle;

							// Set the new info:
							sort_kanji_by[PRIMARY].window_id = wParam;
							sort_kanji_by[PRIMARY].index = header->iItem;
							sort_kanji_by[PRIMARY].sortstyle = ASCENDING;

							// Give it an icon: (primary)
							HDITEM item;
								item.mask = HDI_BITMAP | HDI_FORMAT;
							Header_GetItem( header->hdr.hwndFrom, header->iItem, &item );
								item.mask = HDI_BITMAP | HDI_FORMAT;
								item.hbm = (sort_kanji_by[PRIMARY].sortstyle==ASCENDING)?bitmap_sort_asc1:bitmap_sort_dsc1;
								item.fmt |= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
							Header_SetItem( header->hdr.hwndFrom, header->iItem, &item );

							// Give the secondary a different icon:
							//HDITEM item;
								item.mask = HDI_BITMAP;
								HWND hwndsecondary = sort_kanji_by[SECONDARY].window_id==WID_KANJI_LIST1?hwnd_kanjilist1:hwnd_kanjilist2;
							Header_GetItem( hwndsecondary, sort_kanji_by[SECONDARY].index, &item );
								item.mask = HDI_BITMAP;
								item.hbm = (sort_kanji_by[SECONDARY].sortstyle==ASCENDING)?bitmap_sort_asc2:bitmap_sort_dsc2;
							Header_SetItem( hwndsecondary, sort_kanji_by[SECONDARY].index, &item );
						}
						// If there are any results, sort them:
						SortKanjiResults(); // (this checks to see if there are any).
						break;
					}
				}
			}// if wparam == WID_KANJI_LIST1 or ~2
			return(0);
		}
		case WM_LBUTTONDOWN: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			// Regardless of anything, this window now has keyboard focus:
			SetFocus( hWnd );

			// Select the entry that he clicked on:
			// If there are no results, do nothing:
			if( kanjidic_results.size() == 0 ) {  break;  }

			int old_kanji_selected = kanji_selected;
			kanji_selected = KanjiWindow_GetEntryFromPoint( mx,my );
			
			if( old_kanji_selected != kanji_selected ) {
				RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
			}
			return(0);
		}
		case WM_LBUTTONDBLCLK: {
			// wParam :: what virtual keys are down.
			// lParam :: LONG( x, y ).
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			SendMessage( hWnd,WM_LBUTTONDOWN,wParam,lParam );
			SendMessage( hWnd,WM_KEYDOWN,VK_RETURN,0 );
			return(0); // An application should return 0 if it processes WM_LBUTTONDBLCLK.
		}
		case WM_RBUTTONDOWN: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			if( thread_searching ) {  return(0);  }
			if( kanjidic_results.size() < 1 ) {  return(0);  }

			SendMessage( hWnd,WM_LBUTTONDOWN,wParam,lParam );

			RECT rcWindow;
				GetWindowRect(hWnd,&rcWindow);

			//fprintf(g_flog,"   x,y (%i,%i)\n",(int)mx,(int)my );

			// Select the item that it's over:
			SendMessage( hWnd, WM_LBUTTONDOWN, wParam,lParam );

			// Make the menu:
			HMENU hMenu = CreatePopupMenu();

			HMENU hmenu_search = CreatePopupMenu();
				AppendMenu( hmenu_search, MF_STRING, 1, L"This kanji" );
				AppendMenu( hmenu_search, MF_STRING, 2, L"Words starting with this kanji" );
				AppendMenu( hmenu_search, MF_STRING, 3, L"Words ending with this kanji" );
				AppendMenu( hmenu_search, MF_STRING, 4, L"Words with this kanji anywhere" );
			AppendMenu( hMenu, MF_POPUP, (UINT_PTR)hmenu_search, L"Search..." );
				AppendMenu( hMenu, MF_SEPARATOR, NULL,NULL );
			HMENU hmenu_copy = CreatePopupMenu();
				AppendMenu( hmenu_copy, MF_STRING, 5, L"This kanji" );
				AppendMenu( hmenu_copy, MF_STRING, 6, L"This kanji's On readings" );
				AppendMenu( hmenu_copy, MF_STRING, 7, L"This kanji's Kun readings" );
				AppendMenu( hmenu_copy, MF_STRING, 8, L"This kanji's Nanori readings" );
				AppendMenu( hmenu_copy, MF_STRING, 9, L"This kanji's Radical readings" );
				AppendMenu( hmenu_copy, MF_STRING, 10, L"This kanji's meanings" );
				AppendMenu( hmenu_copy, MF_STRING, 11, L"All info on this kanji" );
			AppendMenu( hMenu, MF_POPUP, (UINT_PTR)hmenu_copy, L"Copy..." );

			int result = TrackPopupMenuEx( hMenu, TPM_LEFTALIGN|TPM_LEFTBUTTON | TPM_RETURNCMD, rcWindow.left+mx,rcWindow.top+my, hWnd, NULL );
			if( result >= 1 && result <= 4 ) {
			// Search for the kanji:
				wstring str;
				if( result == 1 ) {
					str = kanjidic_results[kanji_selected].kanji;
				} else if( result == 2 ) {
					str = kanjidic_results[kanji_selected].kanji;
					str += L"_";
				} else if( result == 3 ) {
					str = L"_";
					str += kanjidic_results[kanji_selected].kanji;
				} else if( result == 4 ) {
					str = L"_";
					str += kanjidic_results[kanji_selected].kanji;
					str += L"_";
				}


				// Set the status bar text:
				SetStatusBarText(L"Searching...\n");


				THREAD_SEARCH ts;
					ts.search_text = &str;
					ts.mode = TS_EDICT_ONLY;

				/*
				// Search for the desired text:
				int search_status = Edict_SearchFor( str );
				//*/


					
					
				// Start searching (thread):
				HANDLE hthread = Load_Thread_Search( &ts );

				// Sit around for a bit until:
				// 1) The thread finishes, or
				// 2) He presses ESCAPE.
				enum {  SEARCH_ABORT, SEARCH_DONE  };
				int wtf = -1;
				MSG msg;
				while( true ) {
					if( !thread_searching ) {  wtf = SEARCH_DONE;  break;  }

					// Check to see if we have an ESCAPE key message.
					PeekMessage( &msg, NULL, 0,0, PM_REMOVE );
					TranslateMessage(&msg);
					if( msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE ) {  wtf = SEARCH_ABORT;  break;  }
					DispatchMessage(&msg);

					Sleep( 10L );
				}
				if( wtf == SEARCH_ABORT ) {
				//////////////////////////
				///   SEARCH ABORTED   ///
				//////////////////////////
					TerminateThread( hthread, 0L );
					thread_searching = false;

					// Set up & redraw the result window:
					// (this also sorts them).
					ResultWindow_SetUpForNewResults();

					// Update the status bar text:
					TCHAR tsearchbar[400];
					swprintf(tsearchbar,L"Search aborted (%i results)",edict_results.size()	);
					SetStatusBarText( tsearchbar );
				} else {
				/////////////////////////////
				///   SEARCH SUCCESSFUL   ///
				/////////////////////////////
					int search_status = thread_search_exitcode;

					// Set up and redraw the results window:
					ResultWindow_SetUpForNewResults();

					// Set the status bar text:
					TCHAR tsearchbar[100];
					swprintf(tsearchbar,L"Search complete (%i results) %s",edict_results.size(),
						search_status&SEARCH_STATUS_TRUNCATED?L"-Results truncated-":L""
					);
					SetStatusBarText( tsearchbar );
				}
			} else if( result == 5 ) {
			// Copy the kanji to the clipboard:
				CopySelectedKanjiInfo( KINFO_KANJI );
			} else if( result == 6 ) {
			// Copy the kanji's ON to the clipboard:
				CopySelectedKanjiInfo( KINFO_ON );
			} else if( result == 7 ) {
			// Copy the kanji's KUN to the clipboard:
				CopySelectedKanjiInfo( KINFO_KUN );
			} else if( result == 8 ) {
			// Copy the kanji's NANORI to the clipboard:
				CopySelectedKanjiInfo( KINFO_NANORI );
			} else if( result == 9 ) {
				// Copy the kanji's RADICAL READING to the clipboard:
				CopySelectedKanjiInfo( KINFO_RADICALREADING );
			} else if( result == 10 ) {
			// Copy the kanji's Meaning to the clipboard:
				CopySelectedKanjiInfo( KINFO_MEANING );
			} else if( result == 11 ) {
			// Copy all of the kanji's info:
				CopySelectedKanjiInfo( KINFO_ALL );
			}
				DestroyMenu( hmenu_search );
				DestroyMenu( hmenu_copy );
			DestroyMenu( hMenu );

			return(0); // An application should return 0 if it processes WM_RBUTTONDOWN.
		}
		case WM_MOUSEMOVE: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			// Set the cursor to the standard arrow:
			SetCursor( cursor_arrow );

			// If we have captured the mouse (and it moves out of bounds),
			// then give it back so we can get the WM_MOUSELEAVE message.
			if( hover_on ) {
				if( mx < rcClient_kanji.left || my < rcClient_kanji.top ||
					 mx >= rcClient_kanji.right || my >= rcClient_kanji.bottom ) {
					ReleaseCapture();
				}
			}

			// Capture the mouse...
			HWND hwnd_focus = GetFocus();
			if( hwnd_focus != hWnd ) {
				HWND hwnd_old = SetFocus( hWnd );
				if( hwnd_old == NULL ) {
					fprintf(g_flog,"SetFocus(): ERROR: %s\n",GetLastError() );
				}
			}


			// When the mouse hovers over an entry, show a pop-up view of it.

			// If there are no results, then we're not doing any pop-ups.
			if( kanjidic_results.size() == 0 ) {  break;  }

			// This is the entry that it's over *now*.
			int hover_entry_test = KanjiWindow_GetEntryFromPoint( mx,my );
			
			// If there's already a pop-up window, and the one that it's
			// over is *different* from that one, then destroy the old one.
			if( hover_on ) {
				if( hover_entry_test != hover_entry ) {
					DestroyWindow( hwnd_hover );
					hover_on = false;
					ReleaseCapture();
				}
			}

			// Now, see if the mouse hovers over an entry.
			if( hover_entry_test != -1 ) {
				TRACKMOUSEEVENT event;
					event.cbSize = sizeof(TRACKMOUSEEVENT);
					event.dwFlags = TME_HOVER;
					event.dwHoverTime = 700; // hover-time (milliseconds).
					event.hwndTrack = hWnd;
				_TrackMouseEvent( &event );
				// Set the hover variables:
				hover_entry = hover_entry_test;
				hover_pos.x = mx;
				hover_pos.y = my;
			}
			return(0);
		}
		case WM_MOUSEHOVER: {
			int mx = LOWORD(lParam), my = HIWORD(lParam);

			// NEGATIVE CASES:
			// * If there's already a hover window up, don't draw it again:
			if( hover_on ) {
				return(0);
			}
			// If the mouse moved since we sent the hover message, NO:
			if( abs( hover_pos.x - mx ) > 10 || abs( hover_pos.y - my ) > 10 ) {
				return(0);
			}
			// If it trailed off into another entry, NO:
			if( KanjiWindow_GetEntryFromPoint(mx,my) != hover_entry ) {
				return(0);
			}

			/////////////////////////////////
			///   MAKE THE HOVER WINDOW   ///
			/////////////////////////////////
			// Otherwise, we have a hover window to create:
			hwnd_hover = CreateKanjiHoverWindow( hover_entry );

			// Capture the mouse so that this window keeps getting mouse input.
			SetCapture( hWnd );

			// Set its position:
			// First, figure out where it is (this uses the parent window for positioning):
			RECT rcWin;
				GetWindowRect( hWnd, &rcWin );
			int newx = rcWin.left+mx, newy = rcWin.top+my;
			// Then, figure out how big it is:
				GetWindowRect( hwnd_hover, &rcWin );
			int neww = rcWin.right-rcWin.left, newh = rcWin.bottom-rcWin.top;

			// Get the area that a full-screen window can use:
			RECT rcDesk;
			SystemParametersInfo( SPI_GETWORKAREA, 0, &rcDesk, 0 );

			// If the pop-up goes too far off, put it back on:
			if( newx+neww >= rcDesk.right ) {  newx = rcDesk.right-neww;  }
			if( newx < 0 ) {  newx = 0;  }

			if( newy+newh >= rcDesk.bottom ) {  newy = rcDesk.bottom-newh;  }
			if( newy < 0 ) {  newy = 0;  }

			// Now, set the position and show it:
			SetWindowPos( hwnd_hover, NULL,
				newx, newy,
				0,0,
				SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW
			);

			// Yes, we have a hover window up.
			hover_on = true;

			// We want to know if the mouse leaves the window, so that we
			// can destroy the hover window then.
			TRACKMOUSEEVENT event;
				event.cbSize = sizeof(TRACKMOUSEEVENT);
				event.dwFlags = TME_LEAVE;
				event.hwndTrack = hWnd;
			_TrackMouseEvent( &event );
			return(0);
		}
		case WM_MOUSELEAVE: {
			if( hover_on ) {
				hover_on = false;
				DestroyWindow( hwnd_hover );
				ReleaseCapture();
			}
			return(0);
		}
		case WM_MOUSEWHEEL: {
		// We use the mousewheel to scroll:
			// Wheel info:
			WORD fwKeys = LOWORD(wParam);
			short zDelta = (short)HIWORD(wParam);

			// Use the following code to obtain the horizontal and vertical position:
			short xPos = LOWORD(lParam); 
			short yPos = HIWORD(lParam);

			if( zDelta < 0 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKEWPARAM(SB_LINEDOWN,0), 0L); 
			} else if( zDelta > 0 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKEWPARAM(SB_LINEUP,0), 0L); 
			}
			//fprintf(g_flog,"zDelta: %i\n",zDelta );
			return(0);
		}
		case WM_KEYDOWN: {
			int entries_that_fit_on_screen = (rcClient_kanji.bottom-rcClient_kanji.top)/(kanji_height);
			int end_kanji = start_kanji + min( entries_that_fit_on_screen, kanjidic_results.size()-1-start_kanji );

			WORD wScrollNotify = -1;
			switch (wParam) {
				// Handle the keys for scrolling:
				case VK_UP: {
					// If an entry is selected, move it up; otherwise, just move the scrollbar:
					if( kanji_selected > 0 ) {
						if( kanji_selected == start_kanji ) {	wScrollNotify = SB_LINEUP;	}
						kanji_selected--;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_LINEUP;
					}
					break;
				}
				case VK_DOWN: {
					// If an entry is selected, move it down; otherwise, just move the scrollbar:
					if( kanji_selected < kanjidic_results.size()-1 ) {
						if( kanji_selected == end_kanji-1 ) {	wScrollNotify = SB_LINEDOWN;	}
						kanji_selected++;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_LINEDOWN;
					}
					break;
				}
				case VK_PRIOR: {//	wScrollNotify = SB_PAGEUP;		break;	}
					// If an entry is selected, move it up; otherwise, just move the scrollbar:
					if( kanji_selected >= 0 ) {
						if( kanji_selected > 0+entries_that_fit_on_screen-1 ) {
							kanji_selected -= entries_that_fit_on_screen;
						} else {
							kanji_selected = 0;
						}
						wScrollNotify = SB_PAGEUP;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_PAGEUP;
					}
					break;
				}
				case VK_NEXT: {//	wScrollNotify = SB_PAGEDOWN;	break;	}
					// If an entry is selected, move it down; otherwise, just move the scrollbar:
					if( kanji_selected >= 0 ) {
						if( kanji_selected < (int)kanjidic_results.size()-entries_that_fit_on_screen ) {
							kanji_selected += entries_that_fit_on_screen;
						} else {
							kanji_selected = kanjidic_results.size()-1;
						}
						wScrollNotify = SB_PAGEDOWN;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_PAGEDOWN;
					}
					break;
				}
				case VK_HOME: {//ScrollNotify = SB_TOP;			break;	}
					// If an entry is selected, move it to the top; otherwise, just move the scrollbar:
					if( kanji_selected >= 0 ) {
						if( kanji_selected != 0 ) {
							kanji_selected = 0;
						}
						wScrollNotify = SB_TOP;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_TOP;
					}
					break;
				}
				case VK_END: {//wScrollNotify = SB_BOTTOM;		break;	}
					// If an entry is selected, move it to the top; otherwise, just move the scrollbar:
					if( kanji_selected >= 0 ) {
						if( kanji_selected != kanjidic_results.size()-1 ) {
							kanji_selected = kanjidic_results.size()-1;
						}
						wScrollNotify = SB_BOTTOM;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_BOTTOM;
					}
					break;
				}
				// Switch windows:
				case VK_TAB: {
					SetFocus( hwnd_searchedit );
					break;
				}
				// Select the top one:
				case VK_RETURN: {
				/* If an entry is selected already, treat it as a double-click.
				   Otherwise, select the top entry on the screen. */

					// If there's already a search going on, stop.
					if( thread_searching ) {  return(0);  }

					if( kanjidic_results.size() > 0 ) {
						if( kanji_selected == -1 ) {
							kanji_selected = start_kanji;

							RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						} else {
						// Do the double-click action on it.
							// Search the kanji (with the default options):
							// Set the status bar text:
							SetStatusBarText(L"Searching...\n");

							/*
							edict_results.clear();
							wstring searchtext = L"";
								searchtext = kanjidic_results[kanji_selected].kanji;
							int search_status = Edict_SearchFor( searchtext );//*/



							wstring searchtext = L"";
								searchtext = kanjidic_results[kanji_selected].kanji;

							THREAD_SEARCH ts;
								ts.search_text = &searchtext;
								ts.mode = TS_EDICT_ONLY;
							// Start searching (thread):
							HANDLE hthread = Load_Thread_Search( &ts );

							// Sit around for a bit until:
							// 1) The thread finishes, or
							// 2) He presses ESCAPE.
							enum {  SEARCH_ABORT, SEARCH_DONE  };
							int wtf = -1;
							MSG msg;
							while( true ) {
								if( !thread_searching ) {  wtf = SEARCH_DONE;  break;  }

								// Check to see if we have an ESCAPE key message.
								PeekMessage( &msg, NULL, 0,0, PM_REMOVE );
								TranslateMessage(&msg);
								if( msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE ) {  wtf = SEARCH_ABORT;  break;  }
								DispatchMessage(&msg);

								Sleep( 10L );
							}
							if( wtf == SEARCH_ABORT ) {
							//////////////////////////
							///   SEARCH ABORTED   ///
							//////////////////////////
								TerminateThread( hthread, 0L );
								thread_searching = false;
			
								// Set up & redraw the result window:
								// (this also sorts them).
								ResultWindow_SetUpForNewResults();
			
								// Update the status bar text:
								TCHAR tsearchbar[400];
								swprintf(tsearchbar,L"Search aborted (%i results)",edict_results.size()	);
								SetStatusBarText( tsearchbar );
							} else {
							/////////////////////////////
							///   SEARCH SUCCESSFUL   ///
							/////////////////////////////
								int search_status = thread_search_exitcode;

								// Set up and redraw the results window:
								ResultWindow_SetUpForNewResults();

								// Set the status bar text:
								TCHAR tsearchbar[100];
								swprintf(tsearchbar,L"Search complete (%i results) %s",edict_results.size(),
									search_status&SEARCH_STATUS_TRUNCATED?L"-Results truncated-":L""
								);
								SetStatusBarText( tsearchbar );
							}
						} // (there are kanji entries...)
					}
					break;
				}
			}
 
			if( wScrollNotify != -1 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKELONG(wScrollNotify, 0), 0L); 
			}
			return(0);
		}// case WM_KEYDOWN
	}
	return( DefWindowProc(hWnd, message, wParam, lParam) );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateKanji_Window( HWND hwndParent, int x,int y, int w,int h ) {
	WNDCLASS wndClass; 
	   //wndClass.style          = CS_SAVEBITS; //CS_HREDRAW | CS_VREDRAW;
		wndClass.style          = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
		wndClass.lpfnWndProc    = WndProc_Kanji_Window;
	   wndClass.cbClsExtra     = 0;
		wndClass.cbWndExtra     = 0;
	   wndClass.hInstance      = g_hinst;
		wndClass.hIcon          = NULL;//LoadIcon(NULL, IDI_APPLICATION);
	   wndClass.hCursor        = NULL;//LoadCursor(NULL, IDC_ARROW);
		wndClass.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
	   wndClass.lpszMenuName   = NULL;
		wndClass.lpszClassName  = TEXT("KanjiWindow");
   
   RegisterClass(&wndClass);
   
   HWND hWnd = CreateWindowEx(
		WS_EX_CLIENTEDGE,
      TEXT("KanjiWindow"),   // window class name
      TEXT("(KanjiWindow)"),  // window caption
      WS_CHILD| WS_VISIBLE | WS_VSCROLL,// window style
      x,y, // (x,y)
      w,h, // (w,h)
      hwndParent,                     // parent window handle
      NULL,                     // window menu handle
      g_hinst,                // program instance handle
      NULL);                    // creation parameters
	  
   //ShowWindow(hWnd, SW_SHOW);
	return(hWnd);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////