// KanjiDic.h | By Doug Manley
///////////////////////////////////////////////////////////
#ifndef KANJIDIC_H
#define KANJIDIC_H
///////////////////////////////////////////////////////////
#include "Conversion.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
Uint32 HexToUint32( const char *text ) {
	Uint32 u = 0;
	Uint32 mod = 1;
	for( int i = strlen(text)-1; i >= 0; i-- ) {
		char c = toupper( text[i] );
		if( c >= '0' && c <= '9' || c >= 'A' && c <= 'F' ) {
			char number = 0; // We don't even need 8 bits...
			if( c >= '0' && c <= '9' ) {
				number = c-'0';
			} else {
				number = 10+c-'A';
			}
			u += (Uint32)number * mod;
				mod *= 16;
		} else {
			break;
		}
	}
	return(u);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
const Uint16 EUC_ENGLISH_START = 0xA3B0;
const Uint16 EUC_ENGLISH_END = 0xA3FA;
const Uint16 EUC_HIRAGANA_START = 0xA4A1;
const Uint16 EUC_HIRAGANA_END = 0xA4F3;
const Uint16 EUC_KATAKANA_START = 0xA5A1;
const Uint16 EUC_KATAKANA_END = 0xA5F6;
///////////////////////////////////////////////////////////
Uint16 EUCToUNICODE( Uint16 c ) {
	Uint16 offset = 0;
	Uint16 u = 0;
	if( c >= EUC_HIRAGANA_START && c <= EUC_HIRAGANA_END ) {
	// HIRAGANA
		offset = c - 0xA4A1; // 0xA4A1 is the first EUC hiragana character.
		u = 0x3041 + offset; // 0x3041 is the first UNICODE hiragana character.
	} else if( c >= EUC_KATAKANA_START && c <= EUC_KATAKANA_END ) {
	// KATAKANA
		offset = c - 0xA5A1; // 0xA5A1 is the first EUC katakana character.
		u = 0x30A1 + offset; // 0x30A0 is the first UNICODE katakana character.
	} else if( c >= EUC_ENGLISH_START && c <= EUC_ENGLISH_END ) {
	// ENGLISH
		offset = c - 0xA3B0;
		u = 0x0030 + offset;
	}
	return( u );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
struct KANJIDIC_Entry {
	Uint16 kanji;
	vector<wstring> on, kun;
	vector<wstring> nanori, radical; // new as of 1.0.2.
	vector<wstring> meanings;
	Uint16 frequency;
	Uint8 strokecount;
};
///////////////////////////////////////////////////////////
vector<KANJIDIC_Entry> kanjidic_results;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
struct KANJIDIC_Index_Entry {
	Uint16 kanji;
	long fileloc;
};
///////////////////////////////////////////////////////////
KANJIDIC_Index_Entry kanjidic_index_entries[7000];
int kanjidic_index_entries_size = 0;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void KANJIDIC_Index_Load() {
// Note: Kanjidic uses JIS X 0208-1990 (or EUC-JP)
// For this reason, we use the U#### entry for our kanji.
	FILE *fkanjidic = fopen( registry.GetSTRING(".\\Dictionaries\\KANJIDIC\\Path"),"rb");
	if( !fkanjidic ) {  return;  }

	int buffer_kanjidic = registry.GetINT(".\\Buffer\\KANJIDIC");

	char *line = new char[buffer_kanjidic];
	int current = 0;
	while( !feof(fkanjidic) ) {
		kanjidic_index_entries[current].fileloc = ftell( fkanjidic );

		// Read a line:
		fgets( line,buffer_kanjidic-1,fkanjidic );
		if( feof(fkanjidic) ) {  break;  }

		// Get rid of the end-line:
		if( line[strlen(line)-1] != '\n' ) {  line[strlen(line)-1] = '\0';  }
		if( line[strlen(line)-1] != '\r' ) {  line[strlen(line)-1] = '\0';  }

		// If it's a comment, keep going:
		if( line[0] == '#' ) {  continue;  }
		


		// Get the kanji:
		// Instead of using the EUC, we'll just use the unicode directly.
		// To do this, we go to the U#### entry and convert it to hexidecimal (unicode).
		char *u_start = strstr( line, " U" );
		if( u_start ) {
			// Copy the #### into u_str[].
			char u_str[5];
				u_str[0] = u_start[2+0];
				u_str[1] = u_start[2+1];
				u_str[2] = u_start[2+2];
				u_str[3] = u_start[2+3];
				u_str[4] = '\0';
			// HexToUint32 uses an UPPERCASE string:
			_strupr( u_str );
			// Convert it to unicode and add the info to the index entry:
			kanjidic_index_entries[current].kanji = HexToUint32( u_str );
		} else {
			fprintf(g_flog,"KANJIDIC_Index_Load(): ERROR: Unicode not found (index: %i)\n",current );
		}


		
		current++;
	}
	delete [] line;
	kanjidic_index_entries_size = current;


	fclose(fkanjidic);
}//*/
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This uses the location of the line to fill in information
   in 'entry'. */
enum { KANA_MODE_ONKUN, KANA_MODE_NANORI, KANA_MODE_RADICAL };
void LoadKANJIDICInfoInto( KANJIDIC_Entry &entry, KANJIDIC_Index_Entry *index ) {
// NOTE: Kanjidic uses JIS X 0208-1990 (EUC-JP).
	if( !index ) {
		fprintf(g_flog,"LoadKANJIDICInfoInto(): ERROR: index is NULL.\n");
		return;
	}

	// Open the kanjidic file:
	FILE *fkanjidic = fopen( registry.GetSTRING(".\\Dictionaries\\KANJIDIC\\Path"),"rb" );
	if( !fkanjidic ) {
		fprintf(g_flog,"LoadKANJIDICInfoInto(): ERROR: File %s not opened.\n",registry.GetSTRING(".\\Dictionaries\\KANJIDIC\\Path") );
		return;
	}
	// Go to the line of the entry that we want:
	fseek( fkanjidic, index->fileloc, 0 );

	int buffer_kanjidic = registry.GetINT(".\\Buffer\\KANJIDIC");

	char *line = new char[buffer_kanjidic];
		// Read a line:
		fgets( line,buffer_kanjidic-1,fkanjidic );
		if( feof(fkanjidic) ) {  return;  }

		// Get rid of the end-line:
		if( line[strlen(line)-1] != '\n' ) {  line[strlen(line)-1] = '\0';  }
		if( line[strlen(line)-1] != '\r' ) {  line[strlen(line)-1] = '\0';  }

		// If it's a comment, keep going:
		if( line[0] == '#' ) {  return;  }
		
		string strline = line;
		

		// Get the kanji:
		int pos = strline.find(' ');
			strline = strline.substr( pos+1, -1 );



		// Set the entry's values:
		entry.kanji = index->kanji;
			// Set some default values (just in case):
			entry.frequency = 0;
			entry.strokecount = 0;

		// Get the other info:
		int kana_mode = KANA_MODE_ONKUN;
		while( strline.size() > 0 ) {

			/* If, for some reason, we end up starting with a space,
				remove the space.
				This also serves to tell the while loop that we're done
				(since all entries *end* with a space). */
			if( strline[0] == ' ' ) {
				if( strline.size() > 1 ) {
					strline = strline.substr(1,-1);
				} else {
					strline = "";
				}
				continue;
			}

			if( strline[0] == 'B' ) { // The Radical (Bushu) number.
			} else if( strline[0] == 'C' ) { // (1) Historical/Classical radical number.
			} else if( strline[0] == 'F' ) { // (1) Frequency (1 (most) -- 2,501 (least)).
				int pos = strline.find(' ');
				if( pos <= 0 ) {
					// ERROR
				} else {
					string strfreq = strline.substr( 1,pos-1 );
					entry.frequency = atoi( strfreq.c_str() );
				}
			} else if( strline[0] == 'G' ) { // (1) Jouyou grade level (1-6,8(general),9(for names)).
			} else if( strline[0] == 'H' ) { // (1) Index in the New Japanese-English Character Dictionary, edited by Jack Halpern.
			} else if( strline[0] == 'N' ) { // (1) Index number in the "Modern Reader's Japanese-English Character Dictionary", edited by Andrew Nelson
			} else if( strline[0] == 'V' ) { // (1) Index number in The New Nelson Japanese-English Character Dictionary, edited by John Haig.
			} else if( strline[0] == 'D' ) { // (1) DX#### (more dictionary codes)
			} else if( strline[0] == 'P' ) { // (1) The SKIP code: P<#>-<#>-<#>
			} else if( strline[0] == 'S' ) { // (1) Stroke Count.
				int pos = strline.find(' ');
				if( pos <= 0 ) {
					// ERROR
				} else {
					string strstrokes = strline.substr( 1,pos-1 );
					entry.strokecount = atoi( strstrokes.c_str() );
				}
			} else if( strline[0] == 'U' ) { // (1) The Unicode encoding (hex).
				// NOTE: we won't need this, since we *use* unicode.
			} else if( strline[0] == 'I' ) { // (1) for The Kanji Dictionary (Tuttle 1996), they are in the form nxnn.n, e.g. 3k11.2
														//     for the Kanji & Kana book (Tuttle), they are in the form INnnnn, where nnnn is the number of the kanji referenced in that book (2nd edition.)
			} else if( strline[0] == 'Q' ) { // (1) Qnnnn.n -- the "Four Corner" code for that kanji.
			} else if( strline[0] == 'M' ) { // (1) MNnnnnnnn and MPnn.nnnn -- the index number and volume.page respectively of the kanji in the 13-volume Morohashi Daikanwajiten
			} else if( strline[0] == 'E' ) { // (1) Ennnn -- the index number used in "A Guide To Remembering Japanese Characters" by Kenneth G. Henshall. There are 1945 kanji with these numbers (i.e. the Jouyou subset.)
			} else if( strline[0] == 'K' ) { // (1) Knnnn -- the index number in the Gakken Kanji Dictionary ("A New Dictionary of Kanji Usage")
			} else if( strline[0] == 'L' ) { // (1) Lnnnn -- the index number used in "Remembering The Kanji" by James Heisig.
			} else if( strline[0] == 'O' ) { // (1) Onnnn -- the index number in "Japanese Names", by P.G. O'Neill. (Weatherhill, 1972) (A warning: some of the numbers end with 'A'. This is how they appear in the book; it is not a problem with the file.)
			} else if( strline[0] == 'W' ) { // (n) Wxxxx -- the romanized form of the Korean reading(s) of the kanji. Most of these kanji have one Korean reading, a few have two or more. The readings are in the (Republic of Korea) Ministry of Education style of romanization.
			} else if( strline[0] == 'Y' ) { // (n) Yxxxxx -- the "Pinyin" of each kanji, i.e. the (Mandarin or Beijing) Chinese romanization. About 6,000 of the kanji have these. Obviously most of the native Japanese kokuji do not have Pinyin, however at least one does as it was taken into Chinese at a later date.
			} else if( strline[0] == 'X' ) { // (n) Xxxxxxx -- a cross-reference code. An entry of, say, XN1234 will mean that the user is referred to the kanji with the (unique) Nelson index of 1234. XJ0xxxx and XJ1xxxx are cross-references to the kanji with the JIS hexadecimal code of xxxx. The `0' means the reference is to a JIS X 0208 kanji, and the `1' references a JIS X 0212 kanji.
			} else if( strline[0] == 'Z' ) { // (n) Zxxxxxx -- a mis-classification code. It means that this kanji is sometimes mis-classified as having the xxxxxx coding
			} else if( strline[0] == '{' ) { // (n) {<English>}
				string stre;

				int pos = strline.find('}');
				if( pos <= 0 ) {
					strline = "";
					// ERROR!
				} else {
					stre = strline.substr( 1,pos-1 );
					strline = strline.substr( pos+1,-1 );
				}

				wstring wstre;
				for( int i = 0; i < stre.size(); i++ ) {
					wstre += (Uint16)stre[i];
				}
				entry.meanings.push_back( wstre );

				continue;
			} else if( strline[0] == 'T' ) { // (n) T1 : where the kanji has special "nanori" (i.e. name) readings, these are preceded by the marker "T1";
														//     T2 : where the kanji is a radical, and the radical name is not already a reading, the radical name is preceded the marker "T2"...
				if( strline[1] == '1' ) {
					kana_mode = KANA_MODE_NANORI;
				} else if( strline[1] == '2' ) {
					kana_mode = KANA_MODE_RADICAL;
				}
			} else if( strline[0] == '-' || (Uint8)strline[0] >= 0xA4 && (Uint8)strline[0] <= 0xA5) {
				// It's a reading...
				// Notes: . (dot) :: JJJ.JJJ :: Indicate the portion of the reading that is okurigana.
				//        - (dash) :: JJJJ- :: It's a prefix.
				//                    -JJJJ : The Reading is a Suffix.

				string strj;

				int pos = strline.find(' ');
				if( pos <= 0 ) {
					strline = "";
				} else {
					strj = strline.substr( 0,pos );
					strline = strline.substr( pos+1,-1 );
				}

				wstring strju = L"";
				// Note: ON readings are in katakana.
				bool katakana = false;
				for( int i = 0; i < strj.size(); i++ ) {
					unsigned char c = strj[i];
					if( (char)c == '.' ) {
						strju += L'.';
					} else if( (char)c == '-' ) {
						strju += L'-';
					} else {
						Uint16 u = 0;
						unsigned char *ptr = (unsigned char*)&u;
						// For some reason, it's this way:
							ptr[0] = (unsigned char)strj[i+1];
							ptr[1] = c;
								i++;

						u = EUCToUNICODE( u );
						if( IsKatakana(u) ) {  katakana = true;  }

						strju += u;
					}
				}
				if( kana_mode == KANA_MODE_ONKUN ) {
					if( katakana ) {
						entry.on.push_back( strju );
					} else {
						entry.kun.push_back( strju );
					}
				} else if( kana_mode == KANA_MODE_NANORI ) {
					entry.nanori.push_back( strju );
				} else if( kana_mode == KANA_MODE_RADICAL ) {
					entry.nanori.push_back( strju );
				}

				continue;
			}


			// Default: Remove the string of the form 'XX...X ':
			int pos = strline.find(' ');
			if( pos <= 0 ) {
				strline = "";
			} else {
				strline = strline.substr( pos+1,-1 );
			}
		}

	delete [] line;
	fclose( fkanjidic );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////