// Main_Window.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H
///////////////////////////////////////////////////////////
#include "Dialog_Options.h"
///////////////////////////////////////////////////////////
enum {
	MI_RESIZE_NONE = 0,
	MI_RESIZE_NS,
	MI_RESIZE_S_EW,
	MI_RESIZE_S_E_NS
};
///////////////////////////////////////////////////////////
#include "Divider.h"

Divider div_main;
///////////////////////////////////////////////////////////
#include "Update.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc_Main_Window( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {
	static int mouseinstatus = 0;
	static POINT ptstart;
	static POINT ptlast;
   
   switch( message ) {
		case WM_CREATE: {
			//fprintf(g_flog,"Main Window: WM_CREATE:\n");
			RECT rc;
				GetClientRect( hWnd, &rc );

			// Make the status bar at the bottom:
			hwnd_main_status = CreateWindowEx( 
		      0L, // no extended styles
		      STATUSCLASSNAME, // status bar
	         TEXT(""), // text 
				WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS, // styles
				rc.bottom-10,0, // (x,y)
				rc.right,10, // (w,h)
			   hWnd, // parent window
		      (HMENU)100, // window ID
	         g_hinst, // instance
				NULL // window data
			);

			// Get the rectangle for the status bar at the bottom:
			RECT rcStatus;
				GetWindowRect( hwnd_main_status, &rcStatus );

			// Resize the client area to account for the status window:
			rc.bottom -= (rcStatus.bottom - rcStatus.top)+1;
			//rc.bottom = 400;


			hwnd_searchwindow = CreateSearch_Window( hWnd, 0,0, 800,100 );
			hwnd_kanjiwindow = CreateKanji_Window( hWnd, 0,0, 100,100 );
			hwnd_resultwindow = CreateResult_Window( hWnd, 0,0, 100,100 );

			hwnd_radicallookup = CreateRadicalLookup_Window( hWnd, 0,0, 100,100 );


			int vert = 145, vert_bottom_horiz = 220;
			if( registry.GetBOOL(".\\Window\\RememberSize") ) {
				vert = registry.GetINT(".\\Window\\Div_vert");
				vert_bottom_horiz = registry.GetINT(".\\Window\\Div_vert_bottom_horiz");

				if( vert < 0 || vert > rc.bottom-rc.top ||
					 vert_bottom_horiz < 0 || vert_bottom_horiz > rc.right-rc.left ) {
					MessageBox(NULL,L"The position settings were incorrect.",L"Warning:",MB_OK );
					vert = 50; vert_bottom_horiz = 220;
				}
			}//*/

			// Set the bounds for the dividers:
			div_main.SetRECT( rc.left,rc.top, rc.right-rc.left, rc.bottom-rc.top );
			// Divide it in half vertically:
			div_main.Divide( DIV_VERTICAL, vert );
				// Set the top half to the search window:
				div_main.GetHalf(DIV_TOP)->SetWindow( hwnd_searchwindow );
				// Divide the bottom half horizontally:
				div_main.GetHalf(DIV_BOTTOM)->Divide( DIV_HORIZONTAL, vert_bottom_horiz );
					// Set the left half to the kanji window:
					div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT)->SetWindow( hwnd_kanjiwindow );
					// Set the right half to the results window:
					div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_RIGHT)->SetWindow( hwnd_resultwindow );
			return(0);
		}
		case WM_SIZE: {
		/* WM_SIZE is used to:
			1) Resize the status window.
			2) Resize the windows controlled by the Divider.
		*/
			// These values apply to the CLIENT AREA:
			int neww = LOWORD(lParam);
			int newh = HIWORD(lParam);

			// Fix the status window automatically:
			SendMessage( hwnd_main_status, WM_SIZE, wParam,lParam );

			// Get the modified sizes:
			RECT rcStatus;
				GetWindowRect( hwnd_main_status, &rcStatus );
			newh -= (rcStatus.bottom - rcStatus.top)+1;

			// Resize the dividers:
			RECT rcMain = div_main.GetRECT();
			div_main.SetRECT( rcMain.left,rcMain.top, neww,newh );
			return(0);
		}
		case WM_SIZING: {
			// Get the new dimentions:
			RECT *rcsizing = (LPRECT)lParam;
				int neww = rcsizing->right;
				int newh = rcsizing->bottom;

			// Get the modified sizes:
			RECT rcStatus;
				GetWindowRect( hwnd_main_status, &rcStatus );
			RECT rc;
				GetClientRect( hWnd, &rc );
			neww = rc.right;
			newh = rc.bottom;
				newh -= (rcStatus.bottom - rcStatus.top)+1;

			// Resize the divider windows:
			RECT rcMain = div_main.GetRECT();
			div_main.SetRECT( rcMain.left,rcMain.top, neww,newh );
			return(0);
		}
		case WM_MOUSEMOVE: {
			int mx = LOWORD(lParam), my = HIWORD(lParam);

			// Set the cursor if we're over a resize bar:
			int result = div_main.SetWindowCursor( mx,my );
			if( result != 0 ) {
				// If we're NOT resizing, then set it to the arrow.
				if( mouseinstatus == 0 ) {
					SetCursor( cursor_arrow );
				}
			}

			// If we're not dragging, then just forget it:
			if( mouseinstatus == 0 ) {  return(0);  }

			POINT pt;
				pt.x = mx; pt.y = my;

			//fprintf(g_flog,"Main Window: WM_MOUSEMOVE:\n");
			//fprintf(g_flog,"   pos( %i, %i )\n",mx,my );

			int dx = pt.x-ptlast.x, dy = pt.y-ptlast.y;
				//fprintf(g_flog,"   dxy( %i, %i )\n",dx,dy );


			/*TCHAR szBuf[1024];
		   wsprintf(szBuf, L"Mouse position: ( %i, %i ) :: Deltaxy: ( %i, %i )", mx,my, dx,dy );
	      SetStatusBarText(szBuf);//*/

			if( mouseinstatus == MI_RESIZE_NS ) {
				int result = div_main.MoveBar( dy );
				if( result != 0 ) {
					fprintf(g_flog,"   * ERROR: result: %i\n",result );
				} else {
					//fprintf(g_flog,"   * Bar moved.\n");
				}
			} else if( mouseinstatus == MI_RESIZE_S_EW ) {
				int result = div_main.GetHalf(DIV_BOTTOM)->MoveBar( dx );
				if( result != 0 ) {
					fprintf(g_flog,"   * ERROR: result: %i\n",result );
				} else {
					//fprintf(g_flog,"   * Bar moved.\n");
				}
			} else if( mouseinstatus == MI_RESIZE_S_E_NS ) {
				int result = div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT)->MoveBar( dy );
				if( result != 0 ) {
					fprintf(g_flog,"   * ERROR: result: %i\n",result );
				} else {
					//fprintf(g_flog,"   * Bar moved.\n");
				}
			}
			ptlast = pt;
			return(0);
		}
		case WM_MOUSELEAVE: {
			// Reset the mouse status when it enters a child window:
			mouseinstatus = 0;
			SetCursor( LoadCursor(NULL,IDC_ARROW) );
			return(0);
		}
		case WM_LBUTTONDOWN: {
			int mx = LOWORD(lParam), my = HIWORD(lParam);
			POINT pt;
				pt.x = mx; pt.y = my;
			// If there's not resizing going on:
			if( mouseinstatus == 0 ) {
				RECT rcBar;
				// If it's in the vertical bar:
				rcBar = div_main.GetDividerBarRECT();
					rcBar.right += rcBar.left;  rcBar.bottom += rcBar.top+1;
				if( PtInRect( &rcBar, pt ) ) {
					mouseinstatus = MI_RESIZE_NS;
						//fprintf(g_flog,"   (In the NS bar...)\n");
				} else {
				// If it's in the horizontal bar:
					rcBar = div_main.GetHalf(DIV_BOTTOM)->GetDividerBarRECT();
						rcBar.right += rcBar.left;  rcBar.bottom += rcBar.top+1;
					if( PtInRect( &rcBar, pt ) ) {
						mouseinstatus = MI_RESIZE_S_EW;
							//fprintf(g_flog,"   (In the S_EW bar...)\n");
					} else if( radicallookup_active == 1 ) {
					// If it's in the vertical bar in the horizontal bar:
						rcBar = div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT)->GetDividerBarRECT();
							rcBar.right += rcBar.left;  rcBar.bottom += rcBar.top+1;
						if( PtInRect( &rcBar, pt ) ) {
							mouseinstatus = MI_RESIZE_S_E_NS;
								//fprintf(g_flog,"   (In the S_E_NS bar...)\n");
						}
					}
				}
				
				// If he clicked something to resize:
				if( mouseinstatus != 0 ) {
					// Set the start point and last point (for delta):
					ptstart.x = mx; ptstart.y = my;

					ptlast.x = mx; ptlast.y = my;

					SetCapture( hWnd );

					TRACKMOUSEEVENT e = { sizeof(TRACKMOUSEEVENT), TME_LEAVE, hWnd, 0 };
					_TrackMouseEvent( &e );
				}
			}
			return(0);
		}
		case WM_LBUTTONUP: {
			int x = LOWORD(lParam), y = HIWORD(lParam);
			// If we were dragging, it's done now, so clean up:
			if( mouseinstatus != 0 ) {
				mouseinstatus = 0;
				SetCapture(NULL);
				SetCursor( LoadCursor(NULL,IDC_ARROW) );
			}
			return(0);
		}
		case WM_COMMAND: {
			/* wParam
					The high-order word specifies the notification code if the message is from a control.
					If the message is from an accelerator, this value is 1.
					If the message is from a menu, this value is zero. 
					
					The low-order word specifies the identifier of the menu item, control, or accelerator. 

				lParam
					Handle to the control sending the message if the message is from a control. Otherwise, this parameter is NULL. 
			//*/
			if( hmenu_dictionaries != NULL ) {
				switch( LOWORD(wParam) ) {
					case 0:
					case DM_DONE: {
						break;
					} 
					default: {
						if( LOWORD(wParam) >= DM_DSTART ) {
							int dict = LOWORD(wParam)-DM_DSTART;
							edict_dictionaries[dict]->on ^= 1;
							wParam = 0;
							lParam = 0;
						}
						//MessageBox(hWnd,L"default:",L"!",MB_OK );
					}
				}
				return(0);
			}
			switch( LOWORD(wParam) ) {			
				case ID_FILE_EXIT: {
					SendMessage( hWnd, WM_CLOSE, 0,0 );
					break;
				}
				case ID_ADD_DICT: {
				// Add a dictionary:

					if( thread_searching ) {
						MessageBox(hWnd,L"Please finish any searches or dictionary-loads before loading a dictionary.\n",L"Note",MB_OK );
						break;
					}


					Uint16 file[1024]; swprintf(file,L"");
					Uint16 fileTitle[1024]; swprintf(fileTitle,L"");
					OPENFILENAME openFileName;
						memset(&openFileName, 0, sizeof(openFileName));               
						openFileName.lStructSize 		 = sizeof(OPENFILENAME);
						openFileName.lpstrFilter 		 = L"All files\0*.*\0\0";
						openFileName.lpstrDefExt 		 = L"";
   					openFileName.lpstrCustomFilter = NULL;
	   				openFileName.nFilterIndex      = 0;
						openFileName.lpstrFile         = file;
						openFileName.nMaxFile			 = sizeof(file);
   					openFileName.lpstrFileTitle    = fileTitle;
						openFileName.nMaxFileTitle		 = sizeof(fileTitle);
   					openFileName.lpstrInitialDir   = NULL;
				   	openFileName.lpstrTitle        = L"Please select a dictionary to use.";
						openFileName.Flags             = OFN_NOCHANGEDIR | OFN_ENABLESIZING | OFN_FILEMUSTEXIST | OFN_EXPLORER;

					if( GetOpenFileName( &openFileName ) == TRUE ) {
					// If the dictionary was a valid file...

						// Check to see if the file has a '.' in it.
						// If it does, pop up a box to make sure that this file is
						// the one that he wants to load.
						// NOTE: EDICT files usually don't have extentions.
						bool loadfile = true;
						if( wcschr( openFileName.lpstrFile, L'.' ) != NULL ) {
							Uint16 wstr[2048];
								swprintf(wstr,L"File:\n%s\n\nThis file possibly has an extention.\nAre you sure that this is what you want?", openFileName.lpstrFile );
							if( MessageBox( hWnd, wstr,L"Note",MB_YESNO ) == IDNO ) {
								loadfile = false;
							}
						}

						// If it's okay to load the file:
						if( loadfile ) {
							// Set the status bar text:
							Uint16 str[2048];
								swprintf(str,L"Loading Dictionary: %s...",openFileName.lpstrFile );
							SetStatusBarText(str);

							// Get the relative path:
							wstring strfile = openFileName.lpstrFile;
							// Get the program path:
							int args; Uint16 **cmdline = CommandLineToArgvW( L"",&args );
							wstring strline = cmdline[0];
							strline = strline.substr( 0,strline.find_last_of(L'\\')+1 );
							int pos = strfile.find( strline.c_str() );
							if( pos >= 0 ) {
								if( strfile.size() > strline.size() ) {
									strfile = strfile.substr( strline.size(),-1 );
									//MessageBox( hWnd, strfile.c_str(),L"!",MB_OK );
								}
							}

							// Make a new dictionary:
							Dictionary_EDICT *edict_current = new Dictionary_EDICT;
								//edict_current->filename = ToString( openFileName.lpstrFile );
								edict_current->filename = ToString( strfile );
								edict_current->displayname = edict_current->filename;
								edict_current->on = true;
								edict_current->index_entries_size = 0;

								/*
								// Load the index:
								EDICT_Index_Load( edict_current );
								//*/




							// Start loading the dictionary (thread):
							HANDLE hthread = Load_Thread_EDICT_Index_Load( edict_current );
		
							// Sit around for a bit until:
							// 1) The thread finishes, or
							// 2) He presses ESCAPE.
							enum {  SEARCH_ABORT, SEARCH_DONE  };
							int wtf = -1;
							MSG msg;
							while( true ) {
								if( !thread_searching ) {  wtf = SEARCH_DONE;  break;  }

								// Check to see if we have an ESCAPE key message.
								PeekMessage( &msg, NULL, 0,0, PM_REMOVE );
								TranslateMessage(&msg);
								if( msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE ) {  wtf = SEARCH_ABORT;  break;  }
								DispatchMessage(&msg);

								Sleep( 10L );
							}
							if( wtf == SEARCH_ABORT ) {
							////////////////////////
							///   LOAD ABORTED   ///
							////////////////////////
								TerminateThread( hthread, 0L );
								thread_searching = false;

								// Free up any memory that the dictionary took up.
								DeleteEDICT( edict_current );

								// Set the status bar text:
								SetStatusBarText( L"Dictionary loading... aborted." );
							} else {
							///////////////////////////
							///   LOAD SUCCESSFUL   ///
							///////////////////////////
								// Add the dictionary to the list:
								edict_dictionaries.push_back( edict_current );

								// Okay, now add it to the Registry:
								registry.CD(".\\Dictionaries\\EDICT");
									int reg_size = registry.GetINT("Number");
									// Make a new key:
									char reg_str[35]; sprintf(reg_str,"%i",reg_size );
									registry.NewKey(reg_str);
									registry.CD(reg_str);
										registry.SetSTRING("Path",edict_current->filename.c_str() );
										registry.SetSTRING("Name",edict_current->displayname.c_str() );
										registry.SetBOOL("On",edict_current->on );
									registry.CD("..");
									registry.SetINT("Number",reg_size+1 );
	
								// Set the status bar text:
								SetStatusBarText( L"Dictionary loaded." );
							}
						}// if( loadfile )
					}// if( GetOpenFileName(...) )
					break;
				}
				case ID_RADICALLOOKUP: {
				// Turn on/off the radical lookup engine:

					// If the radical information hasn't been loaded, yet, stop.
					if( !radical_info ) {  return(0);  }

					radicallookup_active ^= 1;

					HMENU hmenu = GetMenu( hWnd );
					if( radicallookup_active == 1 ) {
					// If Radical Lookup was just turned on:
						CheckMenuItem( hmenu, ID_RADICALLOOKUP, MF_BYCOMMAND|MF_CHECKED );

						// Get the divider of the kanji window:
						Divider *div_S_E = div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT);
						// Divide the kanji window (bottom 1/3 is the radical window).
						RECT rcDiv = div_S_E->GetRECT();
						div_S_E->Divide( DIV_VERTICAL, (rcDiv.bottom-rcDiv.top)*2/3 );
						div_S_E->GetHalf(DIV_TOP)->SetWindow( hwnd_kanjiwindow );
						div_S_E->GetHalf(DIV_BOTTOM)->SetWindow( hwnd_radicallookup );
					} else {
					// If Radical Lookup was just turned off:
						CheckMenuItem( hmenu, ID_RADICALLOOKUP, MF_BYCOMMAND|MF_UNCHECKED );

						// Get the divider of the kanji window:
						Divider *div_S_E = div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT);
						div_S_E->Divide( DIV_NONE, 0 );
						div_S_E->SetWindow( hwnd_kanjiwindow );
						ShowWindow( hwnd_radicallookup, SW_HIDE );
					}
					return(0);
				}
				case ID_DICTIONARIES: {
					RECT rcWindow;
						GetWindowRect( hWnd, &rcWindow );
					POINT pt;
						GetCursorPos( &pt );
					hmenu_dictionaries = CreateDictionaryMenu( hWnd );
					int result = TrackPopupMenuEx( hmenu_dictionaries, TPM_LEFTALIGN|TPM_LEFTBUTTON, pt.x,pt.y, hWnd, NULL );
						DestroyMenu( hmenu_dictionaries );
						hmenu_dictionaries = NULL;
					return(0);
				}
				case ID_OPTIONS_SETTINGS: {
					CreateDialog_Options( hWnd );
					break;
				}
				case ID_OPTIONS_CHECKFORUPDATES: {
					// Do NOT do this if a thread is going:
					if( thread_searching ) {
						MessageBox(hWnd,L"Please wait for any searches or dictionary-loadings to complete\nbefore attempting to update.",L"Note",MB_OK );
						break;
					}
					Update();
					break;
				}
				case ID_HELP_ABOUT: {
					if( window_about_open == false ) {
						// Get the area that a full-screen window can use:
						RECT rcDesk;
							SystemParametersInfo( SPI_GETWORKAREA, 0, &rcDesk, 0 );
						int w = 400, h = 400;
						int x = (rcDesk.right-rcDesk.left)/2-w/2, y = (rcDesk.bottom-rcDesk.top)/2-h/2;

						HWND hwnd_about = CreateWindow_About( hWnd, x,y, w,h );
						ShowWindow( hwnd_about, SW_SHOW );
					}
					break;
				}
			}//*/
			return(0);
		}
		case WM_MENURBUTTONUP: {
		/*	wParam
				Specifies the position of the item when the mouse was released. 
			lParam
				Handle to the menu containing the item.//*/
			if( lParam == (LPARAM)hmenu_dictionaries ) {
				int id = GetMenuItemID( hmenu_dictionaries, wParam );
				if( id >= DM_DSTART && id <= DM_DSTOP ) {
					int state = GetMenuState( hmenu_dictionaries, wParam, MF_BYPOSITION );
					if( state&MF_CHECKED ) {
						CheckMenuItem( hmenu_dictionaries, wParam, MF_BYPOSITION );
						edict_dictionaries[id-DM_DSTART]->on = false;
					} else {
						CheckMenuItem( hmenu_dictionaries, wParam, MF_BYPOSITION|MF_CHECKED );
						edict_dictionaries[id-DM_DSTART]->on = true;
					}
					EnableMenuItem( hmenu_dictionaries, wParam, MF_BYPOSITION|MF_GRAYED );
					EnableMenuItem( hmenu_dictionaries, wParam, MF_BYPOSITION|MF_ENABLED );


					// Update the registry:
					int dict = id-DM_DSTART;
					registry.CD(".\\Dictionaries\\EDICT");
						char path[35]; sprintf(path,"%i",dict );
						registry.CD( path );
							registry.SetBOOL( "On", edict_dictionaries[dict]->on );
						registry.CD("..");
					registry.CD("..");
				} else if( id == DM_ALL ) {
					for( int d = 0; d < edict_dictionaries.size(); d++ ) {
						CheckMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_CHECKED );
						edict_dictionaries[d]->on = true;
						
						EnableMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_GRAYED );
						EnableMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_ENABLED );

						// Update the registry:
						int dict = d;
						registry.CD(".\\Dictionaries\\EDICT");
							char path[35]; sprintf(path,"%i",dict );
							registry.CD( path );
								registry.SetBOOL( "On", edict_dictionaries[dict]->on );
							registry.CD("..");
						registry.CD("..");
					}
				} else if( id == DM_NONE ) {
					for( int d = 0; d < edict_dictionaries.size(); d++ ) {
						CheckMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND );
						edict_dictionaries[d]->on = false;

						EnableMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_GRAYED );
						EnableMenuItem( hmenu_dictionaries, DM_DSTART+d, MF_BYCOMMAND|MF_ENABLED );

						// Update the registry:
						int dict = d;
						registry.CD(".\\Dictionaries\\EDICT");
							char path[35]; sprintf(path,"%i",dict );
							registry.CD( path );
								registry.SetBOOL( "On", edict_dictionaries[dict]->on );
							registry.CD("..");
						registry.CD("..");
					}
				}
			}
			return(0);
		}
		case WM_CLOSE:
    	case WM_QUIT:
    	case WM_DESTROY: {  // message: window being destroyed
         PostQuitMessage(0);
         return(0); // ~"yatta!"
      }
   }
   return( DefWindowProc(hWnd, message, wParam, lParam) );
} // WndProc
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateMain_Window( HWND hwndParent, int x,int y, int w,int h, int show = SW_SHOW ) {
   WNDCLASS wndClass;
	   wndClass.style          = CS_HREDRAW | CS_VREDRAW; //CS_SAVEBITS;
		wndClass.lpfnWndProc    = WndProc_Main_Window;
	   wndClass.cbClsExtra     = 0;
		wndClass.cbWndExtra     = 0;
	   wndClass.hInstance      = g_hinst;
		wndClass.hIcon          = LoadIcon( g_hinst,MAKEINTRESOURCE(IDI_PROGRAM) ); //LoadIcon(NULL, IDI_APPLICATION);
	   wndClass.hCursor        = NULL;//LoadCursor(NULL, IDC_ARROW);
		wndClass.hbrBackground  = //(HBRUSH)GetStockObject(WHITE_BRUSH);
										  //(HBRUSH)GetStockObject(BLACK_BRUSH);
										  //(HBRUSH)GetStockObject(LTGRAY_BRUSH);
										  GetSysColorBrush(COLOR_MENU);
										  //GetSysColorBrush(COLOR_WINDOW);
	   wndClass.lpszMenuName   = MAKEINTRESOURCE(IDR_MENU_MAIN); //NULL;
		wndClass.lpszClassName  = TEXT("JLookUp");
   
	if( RegisterClass(&wndClass) == 0 ) {
		MessageBox(NULL,L"Error registering window class.",L"Error",MB_OK );
		exit(-1);
	}

	Uint16 strwindow[1024];
		swprintf(strwindow,L"JLookUp %s",version_info );
   
   HWND hWnd = CreateWindowEx(
		0,
      TEXT("JLookUp"), // window class name
		strwindow, // Window title
      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, // window style
      x,y,
		w,h,
      hwndParent, // parent window handle
      NULL, // window menu handle
      g_hinst, // program instance handle
      NULL // creation parameters
	);

	if( !hWnd ) {
		MessageBox(NULL,L"Error opening the main window.",L"Error",MB_OK );
		exit(-1);
	}
	  
   ShowWindow( hWnd, show );
	return(hWnd);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////