// RadicalLookup_Window.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef RADICALLOOKUP_WINDOW_H
#define RADICALLOOKUP_WINDOW_H
///////////////////////////////////////////////////////////
// Important variable: This tells the application if
// Radical Lookup is ON or OFF... 0=off; 1=on.
int radicallookup_active = 0;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#include "Radical.h"
///////////////////////////////////////////////////////////
// Colors for the Radical Window:
Color color_rad_bg( 255,255,255 ); // Background color for the radicals.
Color color_rad_rad( 0,0,0 ); // Color of the radical.
Color color_rad_select( 64,64,64 ); // Color of the highlight box.

// Fonts to use:
int hfont_radical_size = 16;
HFONT hfont_radical = CreateFont(hfont_radical_size, 0,0,0,0,0,0,0,0,0,0,0,0,L"MS Mincho" );
///////////////////////////////////////////////////////////
int radical_size = 0;
	int radical_size_padding = 3;

int start_radical_row = 0;

// These are updated automatically:
int rads_that_fit_on_screen_x = 0;
int rads_that_fit_on_screen_y = 0;
// These are updated by the GetStartAndEndRadicals() function:
int start_radical = 0;
int end_radical = 0;
// Updates the 
void GetStartAndEndRadicals() {
	start_radical = start_radical_row*rads_that_fit_on_screen_x;
	end_radical = start_radical + min( rads_that_fit_on_screen_y*rads_that_fit_on_screen_x, radical_info_size-1-start_radical );
}

int highlighted_radical = -1;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void Radical_Search() {
	// Set the status bar text:
	SetStatusBarText(L"Searching...\n");

	bool first_rad = true;
	wstring kanji = L"";
	for( int r = 0; r < radical_info_size; r++ ) {
		// If the radical is selected for searching:
		if( radical_info[r].status == RAD_SELECTED ) {
			// If it's the first time, make the pool of kanji
			// equal to the first radical's kanji:
			if( first_rad ) {
				first_rad = false;
				kanji = radical_info[r].relatedkanji;
			} else {
			// Otherwise,
			// Keep only the kanji that both share:
				wstring newkanji = L"";
				wstring rkanji = radical_info[r].relatedkanji;
				for( int k = 0; k < kanji.size(); k++ ) {
					// If the kanji is shared:
					if( rkanji.find( kanji[k] ) != -1 ) {
						// Only add it if we haven't added it yet:
						if( newkanji.find( kanji[k] ) == -1 ) {
							newkanji += kanji[k];
						}
					}
				}
				kanji = newkanji;
			}
		}
	}

	kanjidic_results.clear();
	for( int i = 0; i < kanji.size(); i++ ) {
		KANJIDIC_Index_Entry *index = NULL;
		for( int k = 0; k < kanjidic_index_entries_size; k++ ) {
			if( kanjidic_index_entries[k].kanji == kanji[i] ) {
				index = &kanjidic_index_entries[k];
				break;
			}
		}
		if( index != NULL ) {
			// Now we want to make sure that we haven't already added it:
			bool okaytoadd = true;
			for( int l = 0; l < kanjidic_results.size(); l++ ) {
				if( kanjidic_results[l].kanji == index->kanji ) {
					okaytoadd = false;
					break;
				}
			}
			// If it's new, then add it:
			if( okaytoadd ) {
				KANJIDIC_Entry entry;
					LoadKANJIDICInfoInto( entry, index );
				kanjidic_results.push_back( entry );
			}
		}
	}
	
	// Set up and redraw the Kanji Window:
	KanjiWindow_SetUpForNewResults();

	TCHAR tsearchbar[100];
	swprintf(tsearchbar,L"Radical Search complete (%i results)",kanjidic_results.size() );
	SetStatusBarText( tsearchbar );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc_RadicalLookup_Window(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	static int tracking_mouse = 0;
	switch( message ) {
		case WM_CREATE: {
			if( hfont_radical ) {
				DeleteObject( hfont_radical );
				hfont_radical = CreateFont(hfont_radical_size, 0,0,0,0,0,0,0,0,0,0,0,0,ToWString(registry.GetSTRING(".\\Kanji\\Fonts\\Kanji\\Font")).c_str() );
			}
			return(0);
		}
		case WM_SIZE: {
			Graphics graphics( GetDC(hWnd) );
				radical_size = Font( GetDC(hWnd),hfont_radical ).GetHeight( &graphics ) + 2*radical_size_padding;

			RECT rcClient;
				GetClientRect( hWnd, &rcClient );

			// Retrieve the dimensions of the client area. 
			int xClient = LOWORD(lParam);
			int yClient = HIWORD(lParam);

				rads_that_fit_on_screen_x = (xClient)/radical_size;
				rads_that_fit_on_screen_y = (yClient)/radical_size+1;
				GetStartAndEndRadicals();


			// Set the vertical scrolling range and page size
			SCROLLINFO si;
				si.cbSize = sizeof(si); 
				si.fMask  = SIF_RANGE | SIF_PAGE;// | SIF_DISABLENOSCROLL; 
				si.nMin   = 0;
				si.nMax   = (rads_that_fit_on_screen_x!=0)?radical_info_size/rads_that_fit_on_screen_x+1 : 0;
				si.nPage  = rads_that_fit_on_screen_y; 
			SetScrollInfo(hWnd, SB_VERT, &si, TRUE);//*/

			// Recalculate the start based on the new size:
				si.fMask  = SIF_ALL;
         GetScrollInfo( hWnd, SB_VERT, &si );
				start_radical_row = si.nPos;

			// This makes resizing smooth:
			SendMessage( hWnd, WM_PAINT, 0,0 );
			return(0);
		}
		case WM_VSCROLL: {
         // Get all the vertial scroll bar information
			SCROLLINFO si; 
				si.cbSize = sizeof(si);
				si.fMask  = SIF_ALL;
         GetScrollInfo( hWnd, SB_VERT, &si );
         // Save the position for comparison later on
         start_radical_row = si.nPos;
         switch( LOWORD(wParam) ) {
				// user clicked the HOME keyboard key
				case SB_TOP: {  si.nPos = si.nMin;  break;  }  
				// user clicked the END keyboard key
				case SB_BOTTOM: {  si.nPos = si.nMax;  break;  }
				// user clicked the top arrow
				case SB_LINEUP: {  si.nPos -= 1;  break;  }
				// user clicked the bottom arrow
				case SB_LINEDOWN: {  si.nPos += 1;  break;  }  
				// user clicked the shaft above the scroll box
				case SB_PAGEUP: {  si.nPos -= si.nPage;  break;  }  
				// user clicked the shaft below the scroll box
				case SB_PAGEDOWN: {  si.nPos += si.nPage;  break;  }  
				// user dragged the scroll box
				case SB_THUMBTRACK: {  si.nPos = si.nTrackPos;  break;  }
         }
         // Set the position and then retrieve it.  Due to adjustments
         //   by Windows it may not be the same as the value set.
				si.fMask = SIF_POS;
         SetScrollInfo( hWnd, SB_VERT, &si, TRUE );
         GetScrollInfo( hWnd, SB_VERT, &si );
         // If the position has changed, scroll window and update it
         if( si.nPos != start_radical_row ) {
				start_radical_row = si.nPos;
				RECT rc;
					GetClientRect( hWnd,&rc );
				RedrawWindow( hWnd, &rc, NULL, RDW_INVALIDATE );
				// RDW_ERASE makes it redraw the BG.
				// RDW_INVALIDATE makes it WM_PAINT.
         }

			GetStartAndEndRadicals();
         return(0);
		}
		case WM_PAINT: {
			HDC hdc;
			PAINTSTRUCT ps;
			hdc = BeginPaint(hWnd, &ps);

				RECT rcClient;
					GetClientRect( hWnd, &rcClient );

				Graphics graphics(hdc);

				// Clear the window:
				{
					SolidBrush brush_radical_bg( color_rad_bg );
					RectF rc;
						rc.X = rcClient.left;
						rc.Y = rcClient.top;
						rc.Width = rcClient.right;
						rc.Height = rcClient.bottom;
					graphics.FillRectangle( &brush_radical_bg, rc );
				}

				Font font_radical( hdc,hfont_radical );

				StringFormat format_radical(StringFormatFlagsNoWrap);
					format_radical.SetTrimming( StringTrimmingNone );
					format_radical.SetAlignment( StringAlignmentCenter );


				// Get the radical information:
				GetStartAndEndRadicals();

				int xoffset = 0;
				int yoffset = 0;
				for( int r = start_radical; r <= end_radical; r++ ) {
					Pen pen_select( color_rad_select );
					SolidBrush brush_radical( color_rad_rad );
					SolidBrush brush_radical_bg( color_rad_bg );
					if( radical_info[r].status == RAD_SELECTED ) {
						InvertColor( pen_select );
						InvertColor( brush_radical_bg );
						InvertColor( brush_radical );
					}
					RectF rc;
					// Draw the background rectangle:
						rc.X = 0+xoffset*radical_size;
						rc.Y = 0+yoffset*radical_size;
						rc.Width = radical_size;
						rc.Height = radical_size;
					graphics.FillRectangle( &brush_radical_bg, rc );
					// If it's highlighted, show that:
					if( highlighted_radical == r ) {
						RectF rch( rc.X,rc.Y, rc.Width-1,rc.Height-1 );
						graphics.DrawRectangle( &pen_select, rch );
						//graphics.DrawRectangle( &pen_select, rc );
					}
					// Text:
						rc.X += radical_size_padding;	rc.Width -= radical_size_padding;
						rc.Y += radical_size_padding;	rc.Height -= radical_size_padding;
					Uint16 strrad[2] = {  radical_info[r].radical, L'\0'  };
					graphics.DrawString( strrad, -1, &font_radical, rc, &format_radical, &brush_radical );

					xoffset++;
					if( xoffset == rads_that_fit_on_screen_x ) {
						xoffset = 0;
						yoffset++;
					}
				}

			EndPaint(hWnd, &ps);
			return(0);
		}//*/
		case WM_ERASEBKGND: {
			return(TRUE);
		}//*/
		case WM_LBUTTONDOWN: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			// If there's some thread action going on, stop.
			if( thread_searching ) {  return(0);  }

			// Get the x position:
			int rad_col = mx/radical_size;
			// Get the y position:
			int rad_row = my/radical_size;

			int rad = start_radical_row*rads_that_fit_on_screen_x + rad_row*rads_that_fit_on_screen_x + rad_col;

			if( rad < radical_info_size ) {
				if( radical_info[rad].status != RAD_SELECTED ) {
					radical_info[rad].status = RAD_SELECTED;
				} else {
					radical_info[rad].status = RAD_NONE;
				}

				RECT rc;
					rc.left = rad_col*radical_size;
					rc.top = rad_row*radical_size;
					rc.right = rc.left + radical_size;
					rc.bottom = rc.top + radical_size;
				RedrawWindow( hWnd, &rc, NULL, RDW_INVALIDATE );

				Radical_Search();
			}
			return(0);
		}
		/*case WM_LBUTTONDBLCLK: {
			// wParam :: what virtual keys are down.
			// lParam :: LONG( x, y ).
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);
			return(0); // An application should return 0 if it processes WM_LBUTTONDBLCLK.
		}//*/
		case WM_RBUTTONDOWN: {
		// Open a menu:
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);


			// If there's some thread action going on, stop.
			if( thread_searching ) {  return(0);  }


			RECT rcWindow;
				GetWindowRect( hWnd, &rcWindow );

			HMENU hmenu = CreatePopupMenu();
			AppendMenu( hmenu, MF_STRING, 1, L"-Search for kanji-" );
				AppendMenu( hmenu, MF_SEPARATOR, NULL,NULL );
			AppendMenu( hmenu, MF_STRING, 2, L"Toggle his radical" );
				AppendMenu( hmenu, MF_SEPARATOR, NULL,NULL );
			AppendMenu( hmenu, MF_STRING, 3, L"(Clear radicals)" );

			int result = TrackPopupMenuEx( hmenu, TPM_LEFTALIGN|TPM_LEFTBUTTON | TPM_RETURNCMD, rcWindow.left+mx,rcWindow.top+my, hWnd, NULL );

			if( result == 1 ) {
				Radical_Search();
			} else if( result == 2 ) {
				SendMessage( hWnd, WM_LBUTTONDOWN, wParam,lParam );
			} else if( result == 3 ) {
				for( int r = 0; r < radical_info_size; r++ ) {
					radical_info[r].status = RAD_NONE;
				}
				Radical_Search();
				RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
			}
			DestroyMenu( hmenu );

			return(0); // An application should return 0 if it processes WM_RBUTTONDOWN.
		}
		case WM_MOUSEMOVE: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			// Set the cursor to the standard arrow:
			SetCursor( cursor_arrow );

			// Capture the mouse...
			HWND hwnd_focus = GetFocus();
			if( hwnd_focus != hWnd ) {
				HWND hwnd_old = SetFocus( hWnd );
				if( hwnd_old == NULL ) {
					fprintf(g_flog,"SetFocus(): ERROR: %s\n",GetLastError() );
				}
			}

			// If it's not tracking the mouse, track it:
			if( tracking_mouse == 0 ) {
				TRACKMOUSEEVENT e = { sizeof(TRACKMOUSEEVENT), TME_LEAVE, hWnd, 0 };
				_TrackMouseEvent( &e );
				tracking_mouse = 1;
			}



			// If there's some thread action going on, stop.
			if( thread_searching ) {  return(0);  }




			// Get the x position:
			int rad_col = mx/radical_size;
			// Get the y position:
			int rad_row = my/radical_size;

			int rad = start_radical_row*rads_that_fit_on_screen_x + rad_row*rads_that_fit_on_screen_x + rad_col;

			if( highlighted_radical != rad ) {
				// Unhighlight the old one:
				{
					int old = highlighted_radical;
					old -= start_radical;
					int old_row = old/rads_that_fit_on_screen_x;
					int old_col = old%rads_that_fit_on_screen_x;
					RECT rc;
						rc.left = old_col*radical_size;
						rc.top = old_row*radical_size;
						rc.right = rc.left + radical_size;
						rc.bottom = rc.top + radical_size;
					RedrawWindow( hWnd, &rc, NULL, RDW_INVALIDATE );
				}

				// Set the new one:
				if( rad < radical_info_size ) {
					highlighted_radical = rad;

					RECT rc;
						rc.left = rad_col*radical_size;
						rc.top = rad_row*radical_size;
						rc.right = rc.left + radical_size;
						rc.bottom = rc.top + radical_size;
					RedrawWindow( hWnd, &rc, NULL, RDW_INVALIDATE );

					// Set the status bar text:
					RadicalInfo *ri = &radical_info[highlighted_radical];
					Uint16 str[1024];
						swprintf(str,L"(Radical Info) :: %c :: Strokes: %i :: %s",ri->radical,ri->strokes, ri->meaning );
					SetStatusBarText( str );
				} else {
					highlighted_radical = -1;
				}
			}
			
			//SendMessage( hWnd, WM_PAINT, 0,0 );
			//RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
			return(0);
		}
		case WM_MOUSELEAVE: {
		// Unhighlight whatever's highlighted and reset it so that
		// there is no highlighted radical.
			// Unhighlight the old one:
			{
				int old = highlighted_radical;
				old -= start_radical;
				int old_row = old/rads_that_fit_on_screen_x;
				int old_col = old%rads_that_fit_on_screen_x;
				RECT rc;
					rc.left = old_col*radical_size;
					rc.top = old_row*radical_size;
					rc.right = rc.left + radical_size;
					rc.bottom = rc.top + radical_size;
				RedrawWindow( hWnd, &rc, NULL, RDW_INVALIDATE );
			}

			highlighted_radical = -1;
			tracking_mouse = 0;

			//RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
			return(0);
		}
		case WM_MOUSEWHEEL: {
		// We use the mousewheel to zoom in on the radicals:
			// Wheel info:
			WORD fwKeys = LOWORD(wParam);
			short zDelta = (short)HIWORD(wParam);

			// Use the following code to obtain the horizontal and vertical position:
			short xPos = LOWORD(lParam); 
			short yPos = HIWORD(lParam);

			// If CTRL is down, resize the font:
			if( GetKeyState(VK_CONTROL)&0x8000 ) {
				if( zDelta < 0 ) {
					if( hfont_radical_size > 6 ) {
						hfont_radical_size--;
					}
				} else if( zDelta > 0 ) {
					hfont_radical_size++;
				}
				DeleteObject( hfont_radical );
				hfont_radical = CreateFont(hfont_radical_size, 0,0,0,0,0,0,0,0,0,0,0,0, ToWString(registry.GetSTRING(".\\Kanji\\Fonts\\Kanji\\Font")).c_str() );

				RECT rc;
					GetClientRect( hWnd, &rc );
				SendMessage( hWnd, WM_SIZE, 0, MAKELONG(rc.right-rc.left,rc.bottom-rc.top) );

				RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
			} else {
			// Otherwise, just scroll:
				if( zDelta < 0 ) {
					SendMessage(hWnd, WM_VSCROLL, MAKEWPARAM(SB_LINEDOWN,0), 0L); 
				} else if( zDelta > 0 ) {
					SendMessage(hWnd, WM_VSCROLL, MAKEWPARAM(SB_LINEUP,0), 0L); 
				}
			}
			return(0);
		}
		case WM_KEYDOWN: {
			WORD wScrollNotify = -1;
			switch (wParam) {
				// Handle the keys for scrolling:
				case VK_UP: {
					wScrollNotify = SB_LINEUP;
					break;
				}
				case VK_DOWN: {
					wScrollNotify = SB_LINEDOWN;
					break;
				}
				case VK_PRIOR: {
					wScrollNotify = SB_PAGEUP;
					break;
				}
				case VK_NEXT: {
					wScrollNotify = SB_PAGEDOWN;
					break;
				}
				case VK_HOME: {
					wScrollNotify = SB_TOP;
					break;
				}
				case VK_END: {
					wScrollNotify = SB_BOTTOM;
					break;
				}
				case VK_OEM_PLUS: {
					hfont_radical_size++;

					DeleteObject( hfont_radical );
					hfont_radical = CreateFont(hfont_radical_size, 0,0,0,0,0,0,0,0,0,0,0,0, ToWString(registry.GetSTRING(".\\Kanji\\Fonts\\Kanji\\Font")).c_str() );

					RECT rc;
						GetClientRect( hWnd, &rc );
					SendMessage( hWnd, WM_SIZE, 0, MAKELONG(rc.right-rc.left,rc.bottom-rc.top) );

					RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					break;
				}
				case VK_OEM_MINUS: {
					if( hfont_radical_size > 6 ) {
						hfont_radical_size--;
					}

					DeleteObject( hfont_radical );
					hfont_radical = CreateFont(hfont_radical_size, 0,0,0,0,0,0,0,0,0,0,0,0, ToWString(registry.GetSTRING(".\\Kanji\\Fonts\\Kanji\\Font")).c_str() );

					RECT rc;
						GetClientRect( hWnd, &rc );
					SendMessage( hWnd, WM_SIZE, 0, MAKELONG(rc.right-rc.left,rc.bottom-rc.top) );

					RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					break;
				}
			}

			if( wScrollNotify != -1 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKELONG(wScrollNotify, 0), 0L); 
			}
			return(0);
		}// case WM_KEYDOWN
	}
	return( DefWindowProc(hWnd, message, wParam, lParam) );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateRadicalLookup_Window( HWND hwndParent, int x,int y, int w,int h ) {
	WNDCLASS wndClass; 
		//wndClass.style          = CS_SAVEBITS; //CS_HREDRAW | CS_VREDRAW;
		wndClass.style          = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
		wndClass.lpfnWndProc    = WndProc_RadicalLookup_Window;
		wndClass.cbClsExtra     = 0;
		wndClass.cbWndExtra     = 0;
		wndClass.hInstance      = g_hinst;
		wndClass.hIcon          = NULL;//LoadIcon(NULL, IDI_APPLICATION);
		wndClass.hCursor        = NULL;//LoadCursor(NULL, IDC_ARROW);
		wndClass.hbrBackground  = (HBRUSH)GetStockObject(GRAY_BRUSH);
	   wndClass.lpszMenuName   = NULL;
		wndClass.lpszClassName  = TEXT("RadicalLookup");
   
	RegisterClass(&wndClass);

	HWND hWnd = CreateWindowEx(
		WS_EX_CLIENTEDGE,
		L"RadicalLookup",
		L"(RadicalLookup)",
		WS_CHILD | WS_VSCROLL,
		x,y,
		w,h,
		hwndParent,
		NULL,
		g_hinst,
		NULL
	);

	//ShowWindow( hWnd,SW_SHOW );
	return( hWnd );
}
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
