// Search.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef SEARCH_H
#define SEARCH_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool StringStartsWith( wstring &str, wstring &text ) {
	if( str.size() >= text.size() ) {
		for( int i = 0; i < text.size(); i++ ) {
			if( str[i] != text[i] ) {
				return(false);
			}
		}
		return(true);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
bool StringEndsWith( wstring &str, wstring &text ) {
	if( str.size() >= text.size() ) {
		for( int i = 0; i < text.size(); i++ ) {
			if( str[str.size()-1-i] != text[text.size()-1-i] ) {
				return(false);
			}
		}
		return(true);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool StringHasInIt( wstring &str, wstring &text ) {
	if( str.size() >= text.size() ) {
		bool matched = false;
		for( int i = 0; i < text.size(); i++ ) {
			if( str[i] == text[0] ) {
				matched = true;
				for( int j = 1; j < text.size(); j++ ) {
					if( i+j < str.size() ) {
						if( str[i+j] != text[j] ) {
							matched=false;
							break;
						}
					} else {
						matched=false;
						break;
					}
				}
				if( matched ) {
					return(true);
				}
			}
		}
		return(false);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool StringsMatch( wstring &entry_text, wstring &text ) {
	bool addit = false;

	if( wc_wcsncmp( text.c_str(), entry_text.c_str(), WC_EXACT, WC_MATCHCASE,WC_WILDCARDS_ON ) == WC_MATCH ) {
		addit = true;
	}

	return(addit);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// Search return codes:
enum {
	SEARCH_STATUS_OKAY = 0,
	SEARCH_STATUS_TRUNCATED = 1,
};
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*		This searches through all active dictionary index
	files for the kana (or the word, depending on the search
	method).
		When if finds a match, it loads all of the information
	from the EDICT into the edict_results vector.
*/
int SearchFor( wstring text ) {
	int search_status = 0;

	if( search_conversion_kana == SEARCH_CONVERSION_KANA_EQUAL ) {
		text = ToKatakana( text );
	}

	// Go through all the dictionaries:
	for( int d = 0; d < edict_dictionaries.size(); d++ ) {
		Dictionary_EDICT *dictionary = edict_dictionaries[d];
		if( !dictionary->on ) {  continue;  }

		Uint16 statustext[1024];
			wsprintf(statustext,L"Searching in dictionary '%S'...",dictionary->displayname.c_str() );
		SetStatusBarText(statustext);

		// Go through all the entries:
		for( int i = 0; i < dictionary->index_entries_size; i++ ) {
			if( edict_results.size() == search_results_max ) {
				search_status |= SEARCH_STATUS_TRUNCATED;
				break;
			}

			wstring entry_kana = dictionary->index_entries[i].kana;
			if( search_conversion_kana == SEARCH_CONVERSION_KANA_EQUAL ) {
				entry_kana = ToKatakana( entry_kana );
			}

			bool addit = false;
			// If we are NOT forcing to look only a words (no kana),
			if( search_forcewordsearch == 0 ) {
				// Find out if we should search through kana or words:
				if( IsKana_WildcardsOkay(text) ) {
					addit = StringsMatch( entry_kana, text );
				} else {
					addit = StringsMatch( wstring(dictionary->index_entries[i].word), text );
				}
			} else {
			// If we ARE forcing a word-search, then do it:
				addit = StringsMatch( wstring(dictionary->index_entries[i].word), text );
			}

			if( addit ) {
				EDICT_Entry entry;
				LoadInfoInto( dictionary, entry, dictionary->index_entries[i] );
					SetInfo( entry );
				edict_results.push_back( entry );
			}
		}// for each entry in the dictionary.
	}// for loop for dictionaries.
	return( search_status );
}//*/
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*		This is the English SearchFor() variation.  It manually
	loads every entry of every dictionary and checks the
	meaning string to see if there's a match.
*/
int SearchFor_English( wstring text ) {
	int search_status = 0;

	// Go through all the dictionaries:
	for( int d = 0; d < edict_dictionaries.size(); d++ ) {
		Dictionary_EDICT *dictionary = edict_dictionaries[d];
		if( !dictionary->on ) {  continue;  }

		Uint16 statustext[1024];
			wsprintf(statustext,L"Searching in dictionary '%S'...",dictionary->displayname.c_str() );
		SetStatusBarText(statustext);

		FILE *fedict = fopen( dictionary->filename.c_str(),"rb");

		// Go through all the entries:
		for( int i = 0; i < dictionary->index_entries_size; i++ ) {
			if( edict_results.size() == search_results_max ) {
				search_status |= SEARCH_STATUS_TRUNCATED;
				break;
			}

			// Note: this only supports case-sensitive searching.
			EDICT_Entry entry;
			LoadInfoInto_Quick( fedict, entry, dictionary->index_entries[i] );
			for( int m = 0; m < entry.meaning.size(); m++ ) {
				int pos = entry.meaning[m].find( text );
				if( pos != (wstring::size_type)-1 ) {
					bool okaytoadd = true;
					// Only if it's START or WHOLE do we care about this:
					if( (search_style_location == SEARCH_STYLE_START || search_style_location == SEARCH_STYLE_WHOLE) &&
						 pos > 0 ) {
						Uint16 u = entry.meaning[m][pos-1];
							//fwprintf(g_flogu,L"   previous char: %c.\n",u );
						// DO NOT add it if it's in the middle of a word:
						if( u >= L'A' && u <= L'Z' ||
							 u >= L'a' && u <= L'z' ) {
							okaytoadd = false;
						}
					}
					// Only if it's END or WHOLE do we care about this:
					if( (search_style_location == SEARCH_STYLE_END || search_style_location == SEARCH_STYLE_WHOLE) &&
						 entry.meaning[m].size() > pos+text.size() ) {
						Uint16 u = entry.meaning[m][pos+text.size()];
						// DO NOT add it if it's in the middle of a word:
						if( u >= L'A' && u <= L'Z' ||
							 u >= L'a' && u <= L'z' ) {
							okaytoadd = false;
						}
					}
					if( okaytoadd ) {
						EDICT_Entry entry_empty;
						LoadInfoInto( fedict, entry_empty, dictionary->index_entries[i] );
							SetInfo( entry_empty );
						edict_results.push_back( entry_empty );
						break;
					}
				}//*/
			}
		}// for each entry in the dictionary.

		// Close the dictionary and move on:
		fclose( fedict );

	}// for loop for dictionaries.

	return( search_status );
}//*/
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
