// Search_Dict.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef SEARCH_DICT_H
#define SEARCH_DICT_H
///////////////////////////////////////////////////////////
// For the sort functions:
#include "EDICT_Sort.h"
#include "KANJIDIC_Sort.h"

// For searching:
#include "Search.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void KanjiWindow_SearchForKanjiIn( wstring wstr ) {
	for( int j = 0; j < wstr.size(); j++ ) {
		Uint16 character = wstr[j];
		// If it's not hiragana or katakana, then it could be kanji:
		if( !( IsHiragana(character) || IsKatakana(character) ) ) {
			KANJIDIC_Index_Entry *index = NULL;
			for( int k = 0; k < kanjidic_index_entries_size; k++ ) {
				if( kanjidic_index_entries[k].kanji == character ) {
					index = &kanjidic_index_entries[k];
					break;
				}
			}
			if( index != NULL ) {
				// Now we want to make sure that we haven't already added it:
				bool okaytoadd = true;
				for( int l = 0; l < kanjidic_results.size(); l++ ) {
					if( kanjidic_results[l].kanji == index->kanji ) {
						okaytoadd = false;
						break;
					}
				}
				// If it's new, then add it:
				if( okaytoadd ) {
					KANJIDIC_Entry entry;
						LoadKANJIDICInfoInto( entry, index );
					kanjidic_results.push_back( entry );
				}
			}//*/
		}
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
int Edict_SearchFor( wstring &searchtext ) {
	if( searchtext.size() == 0 ) {  return(-1);  }

	// Check for special things:
	int old_search_style_location = search_style_location;

	// See if the search is an English search:
	bool search_english = false;
	if( searchtext.find(L"EN:") == (wstring::size_type)0 ) {
		searchtext = searchtext.substr( 3,-1 );
			search_english = true;
		// Note:
		// English search does NOT support wildcards, so we'll
		// need to process them the old way:
		if( searchtext[searchtext.size()-1] == L'_' && searchtext[0] == L'_' ) {
			search_style_location = SEARCH_STYLE_MIDDLE;
			searchtext = searchtext.substr( 1,searchtext.size()-2 );
		} else if( searchtext[0] == L'_' ) {
			search_style_location = SEARCH_STYLE_END;
			searchtext = searchtext.substr( 1,searchtext.size()-1 );
		} else if( searchtext[searchtext.size()-1] == L'_' ) {
			search_style_location = SEARCH_STYLE_START;
			searchtext = searchtext.substr( 0,searchtext.size()-1 );
		}
	}

	// If we're NOT searching in English, then add the wildcards:
	if( !search_english ) {
		if( search_style_location == SEARCH_STYLE_MIDDLE ) {
			//searchtext = L"_" + searchtext + L"_";
			if( searchtext[searchtext.size()-1] != L'_' ) {
				searchtext = searchtext + L"_";
			}
			if( searchtext[0] != L'_' ) {
				searchtext = L"_" + searchtext;
			}
		} else if( search_style_location == SEARCH_STYLE_START ) {
			if( searchtext[searchtext.size()-1] != L'_' ) {
				searchtext = searchtext + L"_";
			}
		} else if( search_style_location == SEARCH_STYLE_END ) {
			if( searchtext[0] != L'_' ) {
				searchtext = L"_" + searchtext;
			}
		}
	}



	// If we're allowing ROMAji conversion:
	if( !search_english && search_conversion_romaji == SEARCH_CONVERSION_ROMAJI_TOKANA ) {
		// RomajiToKana converts the romaji to kana:
		searchtext = RomajiStringToKana( searchtext );
	}

	// Cap the search, if necessary:
	// (get the maximum number of results allowed):
	Uint16 strresults[100];
	SendMessage( hwnd_searchrebar_edit_resultsize,WM_GETTEXT,99,(LPARAM)strresults );
	search_results_max = _wtoi( strresults );


	// Actually do the search:
	int search_status;
	if( !search_english ) {
		search_status = SearchFor( searchtext );
	} else {
		search_status = SearchFor_English( searchtext );
	}
	// Sort the results:
	SortResults();

	// Put the style the way it used to be...
	search_style_location = old_search_style_location;

	return( search_status );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This function should be called after a search is done.
   It:
	* Sorts the kanji results.
	* Clears the selected kanji.
	* Resets the scroll bar.
	* Tells it to redraw the window. */
void KanjiWindow_SetUpForNewResults() {
	// Sort the results:
	SortKanjiResults();

	// Reset the selection:
	kanji_selected = -1;

	// Set the bar's position:
	SCROLLINFO si;
		si.cbSize = sizeof(si); 
		si.fMask  = SIF_POS;
		si.nPos   = 0;
	SetScrollInfo( hwnd_kanjiwindow, SB_VERT, &si, TRUE);//*/

	// Redraw the kanji window:
	// Make sure that the scrollbar is correct:
	RECT rc;
		GetClientRect( hwnd_kanjiwindow,&rc );
	SendMessage( hwnd_kanjiwindow,WM_SIZE, 0,MAKELONG(rc.right,rc.bottom) );
	// Redraw it:
	RedrawWindow( hwnd_kanjiwindow, &rc, NULL, RDW_INVALIDATE );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This function should be called after a search is done.
   It:
	* Sorts the edict results.
	* Clears the selected entries.
	* Resets the scroll bar.
	* Tells it to redraw the window. */
void ResultWindow_SetUpForNewResults() {
	// Sort the results:
	SortResults();

	// Redraw the result window:
	entry_selected = -1; // none is selected, now.

	// Set the bar's position:
	SCROLLINFO si;
		si.cbSize = sizeof(si); 
		si.fMask  = SIF_POS;
		si.nPos   = 0;
	SetScrollInfo( hwnd_resultwindow, SB_VERT, &si, TRUE);//*/

	// Make sure that the scrollbar is correct:
	RECT rc;
		GetClientRect( hwnd_resultwindow,&rc );
	SendMessage( hwnd_resultwindow,WM_SIZE, 0,MAKELONG(rc.right,rc.bottom) );
	// Redraw it:
	RedrawWindow( hwnd_resultwindow, &rc, NULL, RDW_INVALIDATE );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////